@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 #ifdef USE_LIBBLURAY
 
 /**
- * @file
+ * @file blurayparser.cc
  * @brief Bluray disk parser implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "ffmpegfs.h"
@@ -383,11 +383,11 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
             return true;
         }
 
-        snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Title [%s]%s.%s",
+        snprintf(title_buf, sizeof(title_buf) - 1, "%02u. Title [%s]%s.%s",
                  title_idx + 1,
                  replace_all(format_duration(duration), ":", "-").c_str(),
                  is_main_title ? "+" : "",
-                 params.m_format[0].fileext().c_str()); // can safely assume this is a video format
+                 ffmpeg_format[0].fileext().c_str()); // can safely assume this is a video format
     }
     else
     {
@@ -399,19 +399,19 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
             return true;
         }
 
-        snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Chapter %03d [%s]%s.%s",
+        snprintf(title_buf, sizeof(title_buf) - 1, "%02u. Chapter %03u [%s]%s.%s",
                  title_idx + 1,
                  chapter_idx + 1,
                  replace_all(format_duration(duration), ":", "-").c_str(),
                  is_main_title ? "+" : "",
-                 params.m_format[0].fileext().c_str()); // can safely assume this is a video format
+                 ffmpeg_format[0].fileext().c_str()); // can safely assume this is a video format
 
     }
 
     std::string filename(title_buf);
 
     LPVIRTUALFILE virtualfile = nullptr;
-    if (!params.m_format[0].is_multiformat())
+    if (!ffmpeg_format[0].is_multiformat())
     {
         virtualfile = insert_file(VIRTUALTYPE_BLURAY, path + filename, statbuf);
     }
@@ -486,13 +486,13 @@ static bool create_bluray_virtualfile(BLURAY *bd, const BLURAY_TITLE_INFO* ti, c
             stream_info(path, &clip->video_streams[parse_find_best_video_stream()], &channels, &sample_rate, &audio, &width, &height, &framerate, &interleaved);
         }
 
-        Logging::trace(virtualfile->m_origfile, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_bit_rate).c_str(), width, height, av_q2d(framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
+        Logging::trace(virtualfile->m_destfile, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_bit_rate).c_str(), width, height, av_q2d(framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
         if (audio > -1)
         {
-            Logging::trace(virtualfile->m_origfile, "Audio %1 channels %2", channels, format_samplerate(sample_rate).c_str());
+            Logging::trace(virtualfile->m_destfile, "Audio %1 channels %2", channels, format_samplerate(sample_rate).c_str());
         }
 
-        transcoder_set_filesize(virtualfile, duration, audio_bit_rate, channels, sample_rate, video_bit_rate, width, height, interleaved, framerate);
+        transcoder_set_filesize(virtualfile, duration, audio_bit_rate, channels, sample_rate, AV_SAMPLE_FMT_NONE, video_bit_rate, width, height, interleaved, framerate);
 
         virtualfile->m_video_frame_count    = static_cast<uint32_t>(av_rescale_q(duration, av_get_time_base_q(), av_inv_q(framerate)));
         virtualfile->m_predicted_size       = static_cast<size_t>(size);
@@ -564,25 +564,25 @@ static int parse_bluray(const std::string & path, const struct stat * statbuf, v
     }
 }
 
-int check_bluray(const std::string & _path, void *buf, fuse_fill_dir_t filler)
+int check_bluray(const std::string & path, void *buf, fuse_fill_dir_t filler)
 {
-    std::string path(_path);
+    std::string _path(path);
     struct stat stbuf;
     int res = 0;
 
-    append_sep(&path);
+    append_sep(&_path);
 
-    if (stat((path + "BDMV/index.bdmv").c_str(), &stbuf) == 0)
+    if (stat((_path + "BDMV/index.bdmv").c_str(), &stbuf) == 0)
     {
-        if (!check_path(path))
+        if (!check_path(_path))
         {
-            Logging::trace(path, "Bluray detected.");
-            res = parse_bluray(path, &stbuf, buf, filler);
-            Logging::trace(path, "Found %1 titles.", res);
+            Logging::trace(_path, "Bluray detected.");
+            res = parse_bluray(_path, &stbuf, buf, filler);
+            Logging::trace(_path, "Found %1 titles.", res);
         }
         else
         {
-            res = load_path(path, &stbuf, buf, filler);
+            res = load_path(_path, &stbuf, buf, filler);
         }
 
         add_dotdot(buf, filler, &stbuf, 0);
