@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
  */
 
 /**
- * @file
+ * @file buffer.cc
  * @brief Buffer class implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "buffer.h"
@@ -105,7 +105,7 @@ bool Buffer::open_file(uint32_t segment_no, uint32_t flags)
     bool isdefaultsize  = false;
     uint8_t *p          = nullptr;
 
-    if (!map_file(m_ci[index].m_cachefile, &m_ci[index].m_fd, &p, &filesize, &isdefaultsize, 0))
+    if (!map_file(m_ci[index].m_cachefile, &m_ci[index].m_fd, &p, &filesize, &isdefaultsize, 0, (flags & CACHE_FLAG_RW) ? true : false))
     {
         return false;
     }
@@ -254,7 +254,7 @@ bool Buffer::init(bool erase_cache)
             bool isdefaultsize  = false;
             uint8_t *p          = nullptr;
 
-            if (!map_file(m_ci[0].m_cachefile_idx, &m_ci[0].m_fd_idx, &p, &filesize, &isdefaultsize, static_cast<off_t>(sizeof(IMAGE_FRAME)) * virtualfile()->m_video_frame_count))
+            if (!map_file(m_ci[0].m_cachefile_idx, &m_ci[0].m_fd_idx, &p, &filesize, &isdefaultsize, static_cast<off_t>(sizeof(IMAGE_FRAME)) * virtualfile()->m_video_frame_count, false))
             {
                 throw false;
             }
@@ -337,7 +337,7 @@ bool Buffer::segment_exists(uint32_t segment_no)
     return file_exists(m_ci[segment_no - 1].m_cachefile);
 }
 
-bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t *filesize, bool *isdefaultsize, off_t defaultsize) const
+bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t *filesize, bool *isdefaultsize, off_t defaultsize, bool truncate) const
 {
     bool success = true;
 
@@ -347,7 +347,7 @@ bool Buffer::map_file(const std::string & filename, int *fd, uint8_t **p, size_t
     {
         struct stat sb;
 
-        *fd = ::open(filename.c_str(), O_CREAT | O_RDWR, static_cast<mode_t>(0644));
+        *fd = ::open(filename.c_str(), O_CREAT | O_RDWR | (truncate ? O_TRUNC : 0), static_cast<mode_t>(0644));
         if (*fd == -1)
         {
             Logging::error(filename, "Error opening cache file: (%1) %2", errno, strerror(errno));
@@ -423,7 +423,7 @@ bool Buffer::unmap_file(const std::string &filename, int * fd, uint8_t **p, size
     int     __fd        = *fd;
 
     // Clear all variables
-    p           = nullptr;
+    *p          = nullptr;
     *filesize   = 0;
     *fd         = -1;
 
@@ -512,9 +512,9 @@ bool Buffer::release(int flags /*= CACHE_CLOSE_NOOPT*/)
     return success;
 }
 
-bool Buffer::remove_cachefile(uint32_t segment_no)
+bool Buffer::remove_cachefile(uint32_t segment_no) const
 {
-    LPCACHEINFO ci = !segment_no ? m_cur_ci : &m_ci[segment_no - 1];
+    LPCCACHEINFO ci = !segment_no ? m_cur_ci : &m_ci[segment_no - 1];
     bool success = remove_file(ci->m_cachefile);
 
     if (!ci->m_cachefile_idx.empty())
@@ -559,34 +559,37 @@ bool Buffer::clear()
 {
     std::lock_guard<std::recursive_mutex> lck (m_mutex);
 
-    if (!segment_count() || m_cur_ci == nullptr || m_cur_ci->m_buffer == nullptr)
-    {
-        errno = EBADF;
-        return false;
-    }
-
     bool success = true;
 
-    m_cur_ci->m_buffer_pos          = 0;
-    m_cur_ci->m_buffer_watermark    = 0;
-    m_cur_ci->m_buffer_size         = 0;
-    m_cur_ci->m_seg_finished        = false;
-
-    // If empty set file size to 1 page
-    long filesize = sysconf (_SC_PAGESIZE);
-
-    if (m_cur_ci->m_fd != -1)
+    for (uint32_t n = 0; n < segment_count(); n++)
     {
-        if (ftruncate(m_cur_ci->m_fd, filesize) == -1)
+        LPCACHEINFO ci = &m_ci[n];
+
+        ci->m_buffer_pos          = 0;
+        ci->m_buffer_watermark    = 0;
+        ci->m_buffer_size         = 0;
+        ci->m_seg_finished        = false;
+
+        if (ci->m_fd != -1)
         {
-            Logging::error(m_cur_ci->m_cachefile, "Error calling ftruncate() to clear the file: (%1) %2 (fd = %3)", errno, strerror(errno), m_cur_ci->m_fd);
-            success = false;
-        }
-    }
+            // If empty set file size to 1 page
+            long filesize = sysconf(_SC_PAGESIZE);
 
-    if (m_cur_ci->m_fd_idx != -1)
-    {
-        memset(m_cur_ci->m_buffer_idx, 0, m_cur_ci->m_buffer_size_idx);
+            if (ftruncate(ci->m_fd, filesize) == -1)
+            {
+                Logging::error(ci->m_cachefile, "Error calling ftruncate() to clear the file: (%1) %2 (fd = %3)", errno, strerror(errno), ci->m_fd);
+                success = false;
+            }
+        }
+        else
+        {
+            remove_file(ci->m_cachefile);
+        }
+
+        if (ci->m_fd_idx != -1)
+        {
+            memset(ci->m_buffer_idx, 0, ci->m_buffer_size_idx);
+        }
     }
 
     return success;
