@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
  */
 
 /**
- * @file
+ * @file cache_entry.h
  * @brief %Cache entry
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef CACHE_ENTRY_H
@@ -52,12 +52,23 @@ private:
      * @param[in] virtualfile - Requesting virtual file.
      */
     explicit Cache_Entry(Cache *owner, LPVIRTUALFILE virtualfile);
+    /**
+     * @brief Copy constructor declared deleted, should use create to get this object to maintain reference count.
+     */
+    Cache_Entry(Cache_Entry &) = delete;
     /**
      * @brief Destroy Cache_Entry object.
      */
     virtual ~Cache_Entry();
 
 public:
+    /**
+     * @brief operator = declared deleted, should use create to get this object to maintain reference count.
+     * @param other - Assignment object
+     * @return Pointer to this
+     */
+    Cache_Entry & operator= (Cache_Entry const & other) = delete;
+
     /**
      * @brief Create a new Cache_Entry object.
      * @param[in] owner - Cache object of owner.
@@ -157,6 +168,16 @@ public:
      * @return Returns the current reference counter.
      */
     int                     ref_count() const;
+    /**
+     * @brief Increment the current reference counter.
+     * @return Returns the current reference counter.
+     */
+    int                     inc_refcount();
+    /**
+     * @brief Decrement the current reference counter.
+     * @return Returns the current reference counter.
+     */
+    int                     decr_refcount();
 
     /**
      * @brief Check if cache entry needs to be recoded
@@ -188,6 +209,27 @@ public:
      */
     unsigned int            read_count() const;
 
+    /**
+     * @brief Get if cache has been finished.
+     * @return Returns true if cache is finished, false if not.
+     */
+    bool                    is_finished() const;
+    /**
+     * @brief Get if cache has been finished, but not completely filled.
+     * @return Returns true if cache is finished, but not completely filled, false if not.
+     */
+    bool                    is_finished_incomplete() const;
+    /**
+     * @brief Get if cache has been finished and filled successfully.
+     * @return Returns true if cache is finished successfully, false if not.
+     */
+    bool                    is_finished_success() const;
+    /**
+     * @brief Get if cache has been finished and with an error.
+     * @return Returns true if cache is finished with error, false if not.
+     */
+    bool                    is_finished_error() const;
+
 protected:
     /**
      * @brief Close buffer object.
@@ -214,13 +256,13 @@ protected:
     Cache *                 m_owner;                        /**< @brief Owner cache object */
     std::recursive_mutex    m_mutex;                        /**< @brief Access mutex */
 
-    int                     m_ref_count;                    /**< @brief Reference counter */
+    volatile int            m_ref_count;                    /**< @brief Reference counter */
 
     LPVIRTUALFILE           m_virtualfile;                  /**< @brief Underlying virtual file object */
 
 public:
     Buffer *                m_buffer;                       /**< @brief Buffer object */
-    bool                    m_is_decoding;                  /**< @brief true while file is decoding */
+    volatile bool           m_is_decoding;                  /**< @brief true while file is decoding */
     std::recursive_mutex    m_active_mutex;                 /**< @brief Mutex while thread is active */
 
     CACHE_INFO              m_cache_info;                   /**< @brief Info about cached object */
