@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 #ifdef USE_LIBDVD
 
 /**
- * @file
+ * @file dvdparser.cc
  * @brief dvdparser class implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "ffmpegfs.h"
@@ -248,7 +248,7 @@ static bool create_dvd_virtualfile(const ifo_handle_t *vts_file, const std::stri
     int pgcnum              = vts_ptt_srpt->title[ttnnum - 1].ptt[chapter_idx].pgcn;
     int pgn                 = vts_ptt_srpt->title[ttnnum - 1].ptt[chapter_idx].pgn;
     const pgc_t *cur_pgc    = vts_file->vts_pgcit->pgci_srp[pgcnum - 1].pgc;
-    AVRational framerate    = { 0, 0 };
+    AVRational framerate;
     int64_t duration        = 0;
     uint64_t size           = 0;
     int interleaved         = 0;
@@ -315,7 +315,7 @@ static bool create_dvd_virtualfile(const ifo_handle_t *vts_file, const std::stri
                         chapter_no,
                         angle_no,
                         replace_all(format_duration(duration), ":", "-").c_str(),
-                        params.m_format[0].fileext().c_str());
+                        ffmpeg_format[0].fileext().c_str());
             }
             else
             {
@@ -323,7 +323,7 @@ static bool create_dvd_virtualfile(const ifo_handle_t *vts_file, const std::stri
                         title_no,
                         chapter_no,
                         replace_all(format_duration(duration), ":", "-").c_str(),
-                        params.m_format[0].fileext().c_str());
+                        ffmpeg_format[0].fileext().c_str());
             }
         }
         else
@@ -335,21 +335,21 @@ static bool create_dvd_virtualfile(const ifo_handle_t *vts_file, const std::stri
                         title_no,
                         angle_no,
                         replace_all(format_duration(duration), ":", "-").c_str(),
-                        params.m_format[0].fileext().c_str());
+                        ffmpeg_format[0].fileext().c_str());
             }
             else
             {
                 snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Title [%s].%s",
                         title_no,
                         replace_all(format_duration(duration), ":", "-").c_str(),
-                        params.m_format[0].fileext().c_str());
+                        ffmpeg_format[0].fileext().c_str());
             }
         }
 
         std::string filename(title_buf);
 
         LPVIRTUALFILE virtualfile = nullptr;
-        if (!params.m_format[0].is_multiformat())
+        if (!ffmpeg_format[0].is_multiformat())
         {
             virtualfile = insert_file(VIRTUALTYPE_DVD, path + filename, statbuf);
         }
@@ -393,13 +393,13 @@ static bool create_dvd_virtualfile(const ifo_handle_t *vts_file, const std::stri
                 video_bit_rate      = static_cast<BITRATE>(size * 8LL * AV_TIME_BASE / static_cast<uint64_t>(duration));   // calculate bitrate in bps
             }
 
-            Logging::trace(virtualfile->m_origfile, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_settings.m_video_bit_rate).c_str(), video_settings.m_width, video_settings.m_height, av_q2d(framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
+            Logging::trace(virtualfile->m_destfile, "Video %1 %2x%3@%<%5.2f>4%5 fps %6 [%7]", format_bitrate(video_settings.m_video_bit_rate).c_str(), video_settings.m_width, video_settings.m_height, av_q2d(framerate), interleaved ? "i" : "p", format_size(size).c_str(), format_duration(duration).c_str());
             if (audio_stream > -1)
             {
-                Logging::trace(virtualfile->m_origfile, "Audio %1 Channels %2", audio_settings.m_channels, audio_settings.m_sample_rate);
+                Logging::trace(virtualfile->m_destfile, "Audio %1 Channels %2", audio_settings.m_channels, audio_settings.m_sample_rate);
             }
 
-            transcoder_set_filesize(virtualfile, duration, audio_settings.m_audio_bit_rate, audio_settings.m_channels, audio_settings.m_sample_rate, video_bit_rate, video_settings.m_width, video_settings.m_height, interleaved, framerate);
+            transcoder_set_filesize(virtualfile, duration, audio_settings.m_audio_bit_rate, audio_settings.m_channels, audio_settings.m_sample_rate, AV_SAMPLE_FMT_NONE, video_bit_rate, video_settings.m_width, video_settings.m_height, interleaved, framerate);
 
             virtualfile->m_video_frame_count    = static_cast<uint32_t>(av_rescale_q(duration, av_get_time_base_q(), av_inv_q(framerate)));
             virtualfile->m_predicted_size       = static_cast<size_t>(size);
@@ -542,25 +542,25 @@ static int parse_dvd(const std::string & path, const struct stat *statbuf, void
     }
 }
 
-int check_dvd(const std::string & _path, void *buf, fuse_fill_dir_t filler)
+int check_dvd(const std::string & path, void *buf, fuse_fill_dir_t filler)
 {
-    std::string path(_path);
+    std::string _path(path);
     struct stat stbuf;
     int res = 0;
 
-    append_sep(&path);
+    append_sep(&_path);
 
-    if (stat((path + "VIDEO_TS.IFO").c_str(), &stbuf) == 0 || stat((path + "VIDEO_TS/VIDEO_TS.IFO").c_str(), &stbuf) == 0)
+    if (stat((_path + "VIDEO_TS.IFO").c_str(), &stbuf) == 0 || stat((_path + "VIDEO_TS/VIDEO_TS.IFO").c_str(), &stbuf) == 0)
     {
-        if (!check_path(path))
+        if (!check_path(_path))
         {
-            Logging::trace(path, "DVD detected.");
-            res = parse_dvd(path, &stbuf, buf, filler);
-            Logging::trace(path, "Found %1 titles.", res);
+            Logging::trace(_path, "DVD detected.");
+            res = parse_dvd(_path, &stbuf, buf, filler);
+            Logging::trace(_path, "Found %1 titles.", res);
         }
         else
         {
-            res = load_path(path, &stbuf, buf, filler);
+            res = load_path(_path, &stbuf, buf, filler);
         }
 
         add_dotdot(buf, filler, &stbuf, 0);
