@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
  */
 
 /**
- * @file
+ * @file ffmpeg_profiles.h
  * @brief FFmpeg encoder profiles
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef FFMPEG_PROFILES_H
@@ -45,6 +45,8 @@ public:
 #define OPT_ALL         0x00000000                      /**< @brief All files */
 #define OPT_AUDIO       0x00000001                      /**< @brief For audio only files */
 #define OPT_VIDEO       0x00000002                      /**< @brief For videos (not audio only) */
+#define OPT_SW_ONLY     0x00000004                      /**< @brief Use this option for software encoding only */
+#define OPT_HW_ONLY     0x00000008                      /**< @brief Use this option for hardware encoding only */
 
     typedef struct PROFILE_OPTION                       /**< @brief Profiles options */
     {
