@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,15 +19,14 @@
  * Version 3 can be found in `/usr/share/common-licenses/GPL-3'.
  */
 
-
 /**
- * @file
+ * @file ffmpeg_transcoder.h
  * @brief FFmpeg transcoder
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef FFMPEG_TRANSCODER_H
@@ -107,7 +106,9 @@ public:
             m_filetype(FILETYPE_UNKNOWN),
             m_filename("unset"),
             m_format_ctx(nullptr),
-            m_pix_fmt(AV_PIX_FMT_NONE)
+            m_pix_fmt(AV_PIX_FMT_NONE),
+            m_audio_start_time(0),
+            m_video_start_time(0)
         {}
 
         FILETYPE                m_filetype;             /**< @brief File type, MP3, MP4, OPUS etc. */
@@ -120,6 +121,9 @@ public:
         AVPixelFormat           m_pix_fmt;              /**< @brief Video stream pixel format */
 
         std::vector<STREAMREF>  m_album_art;            /**< @brief Album art stream */
+
+        int64_t                 m_audio_start_time;     /**< @brief Start time of the audio stream in input audio stream time base units, may be 0 */
+        int64_t                 m_video_start_time;     /**< @brief Start time of the video stream in input video stream time base units, may be 0 */
     };
 
     // Output file
@@ -129,15 +133,27 @@ public:
             m_audio_pts(0),
             m_video_pts(0),
             m_last_mux_dts(AV_NOPTS_VALUE)
-        {}
+        {
+            memset(&m_id3v1, 0, sizeof(m_id3v1));
+        }
 
-        int64_t                 m_audio_pts;            /**< @brief Global timestamp for the audio frames */
-        int64_t                 m_video_pts;            /**< @brief Global timestamp for the video frames */
+        int64_t                 m_audio_pts;            /**< @brief Global timestamp for the audio frames in output audio stream time base units  */
+        int64_t                 m_video_pts;            /**< @brief Global timestamp for the video frames in output video stream time base units  */
         int64_t                 m_last_mux_dts;         /**< @brief Last muxed DTS */
 
         ID3v1                   m_id3v1;                /**< @brief mp3 only, can be referenced at any time */
     };
 
+    typedef std::map<AVHWDeviceType, AVPixelFormat> DEVICETYPE_MAP;     /**< @brief Map device types to pixel formats */
+
+    typedef enum HWACCELMODE                            /**< @brief Currently active hardware acceleration mode */
+    {
+        HWACCELMODE_NONE,                               /**< @brief Hardware acceleration not active */
+        HWACCELMODE_ENABLED,                            /**< @brief Hardware acceleration is active */
+        HWACCELMODE_FALLBACK                            /**< @brief Hardware acceleration selected, but fell back to software */
+
+    } HWACCELMODE;
+
 public:
     /**
      * Construct FFmpeg_Transcoder object
@@ -181,7 +197,7 @@ public:
      */
     int                         process_single_fr(int & status);
     /**
-     * Encode any remaining PCM data to the given Buffer. This should be called
+     * Encode any remaining PCM data to the given buffer. This should be called
      * after all input data has already been passed to encode_pcm_data().
      * @return On success returns 0; on error negative AVERROR.
      */
@@ -199,12 +215,12 @@ public:
      * @brief Get the file duration.
      * @return Returns the file in AV_TIME_BASE units.
      */
-    int64_t                     duration();
+    int64_t                     duration() const;
     /**
      * @brief Try to predict the recoded file size. This may (better will surely) be inaccurate.
      * @return Predicted file size in bytes.
      */
-    size_t                      predicted_filesize();
+    size_t                      predicted_filesize() const;
     /**
      * @brief Get the number of video frames in file.
      * @return On success, returns the number of frames; on error, returns 0 (calculation failed or no video source file).
@@ -224,27 +240,28 @@ public:
      * @brief Return source filename. Must be implemented in child class.
      * @return Returns filename.
      */
-    virtual const char *        filename() const;
+    virtual const char *        filename() const override;
     /**
      * @brief Return destination filename. Must be implemented in child class.
      * @return Returns filename.
      */
-    virtual const char *        destname() const;
+    virtual const char *        destname() const override;
     /**
      * @brief Predict audio file size. This may (better will surely) be inaccurate.
-     * @param[in] filesize - Predicted file size in bytes.
-     * @param[in] codec_id - Target codec ID
+     * @param[out] filesize - Predicted file size in bytes, including audio stream size.
+     * @param[in] codec_id - Target codec ID.
      * @param[in] bit_rate - Target bit rate.
      * @param[in] duration - File duration.
      * @param[in] channels - Number of channels in target file.
      * @param[in] sample_rate - Sample rate of target file.
+     * @param[in] sample_format - Selected sample format
      * @return On success, returns true; on failure, returns false.
      */
-    static bool                 audio_size(size_t *filesize, AVCodecID codec_id, BITRATE bit_rate, int64_t duration, int channels, int sample_rate);
+    static bool                 audio_size(size_t *filesize, AVCodecID codec_id, BITRATE bit_rate, int64_t duration, int channels, int sample_rate, AVSampleFormat sample_format);
     /**
      * @brief Predict video file size. This may (better will surely) be inaccurate.
-     * @param[in] filesize - Predicted file size in bytes.
-     * @param[in] codec_id - Target codec ID
+     * @param[out] filesize - Predicted file size in bytes, including video stream size.
+     * @param[in] codec_id - Target codec ID.
      * @param[in] bit_rate - Target bit rate.
      * @param[in] duration - File duration.
      * @param[in] width - Target video width.
@@ -254,6 +271,13 @@ public:
      * @return On success, returns true; on failure, returns false.
      */
     static bool                 video_size(size_t *filesize, AVCodecID codec_id, BITRATE bit_rate, int64_t duration, int width, int height, int interleaved, const AVRational & framerate);
+    /**
+     * @brief Predict overhead in file size. This may (better will surely) be inaccurate.
+     * @param[out] filesize - Predicted file size in bytes, including overhead.
+     * @param filetype - File type: MP3, TS etc.
+     * @return On success, returns true; on failure, returns false.
+     */
+    static bool                 total_overhead(size_t *filesize, FILETYPE filetype);
     /**
      * @brief Closes the output file of open and reports lost packets. Can safely be called again after the file was already closed or if the file was never open.
      * @return Returns true if the output file was closed, false if it was not upon upon calling this function.
@@ -301,6 +325,14 @@ public:
      * @brief Flush FFmpeg's input buffers
      */
     void                        flush_buffers();
+    /**
+     * @brief Flush delayed audio packets, if there are any
+     */
+    int                         flush_delayed_audio();
+    /**
+     * @brief Flush delayed video packets, if there are any
+     */
+    int                         flush_delayed_video();
 
 protected:
     /**
@@ -311,6 +343,18 @@ protected:
      * @return On success returns 0; on error negative AVERROR.
      */
     int                         open_bestmatch_decoder(AVCodecContext **avctx, int *stream_idx, AVMediaType type);
+    /**
+     * @brief Determine the hardware pixel format for the codec, if applicable.
+     * @param codec - Input codec used
+     * @param dev_type - Hardware device type
+     * @param use_device_ctx - If true checks for pix format if using a hardware device context, for a pix format using a hardware frames context otherwise.
+     * @return Returns hardware pixel format, or AV_PIX_FMT_NONE if not applicable.
+     */
+#if IF_DECLARED_CONST
+    AVPixelFormat               get_hw_pix_fmt(const AVCodec *codec, AVHWDeviceType dev_type, bool use_device_ctx) const;
+#else // !IF_DECLARED_CONST
+    AVPixelFormat               get_hw_pix_fmt(AVCodec *codec, AVHWDeviceType dev_type, bool use_device_ctx) const;
+#endif // !IF_DECLARED_CONST
     /**
      * @brief Open codec context for stream_idx.
      * @param[out] avctx - Newly created codec context
@@ -319,16 +363,20 @@ protected:
      * @param[in] type - Type of media: audio or video.
      * @return On success returns 0; on error negative AVERROR.
      */
+#if IF_DECLARED_CONST
+    int                         open_decoder(AVCodecContext **avctx, int stream_idx, const AVCodec *input_codec, AVMediaType type);
+#else // !IF_DECLARED_CONST
     int                         open_decoder(AVCodecContext **avctx, int stream_idx, AVCodec *input_codec, AVMediaType type);
+#endif // !IF_DECLARED_CONST
     /**
      * @brief Open output frame set. Data will actually be written to buffer and copied by FUSE when accessed.
-     * @param buffer[in] - Stream buffer to operate on
+     * @param[in] buffer - Stream buffer to operate on
      * @return On success returns 0; on error negative AVERROR.
      */
     int                         open_output_frame_set(Buffer *buffer);
     /**
      * @brief Open output file. Data will actually be written to buffer and copied by FUSE when accessed.
-     * @param buffer[in] - Stream buffer to operate on
+     * @param[in] buffer - Stream buffer to operate on
      * @return On success returns 0; on error negative AVERROR.
      */
     int                         open_output(Buffer *buffer);
@@ -388,20 +436,31 @@ protected:
      * @brief Open an output file and the required encoder.
      * Also set some basic encoder parameters.
      * Some of these parameters are based on the input file's parameters.
-     * @param buffer[in] - Stream buffer to operate on
+     * @param[in] buffer - Stream buffer to operate on
      * @return On success returns 0; on error negative AVERROR.
      */
     int                         open_output_filestreams(Buffer *buffer);
+    /**
+     * @brief Safely copy a tag to a target buffer. If the input buffer size
+     * is larger than output the data will be truncated to avoid overruns.
+     * The function never appends a /0 terminator.
+     * @param[out] out - Target buffer
+     * @param[in] in - Input buffer
+     * @return Constant pointer to target buffer.
+     */
+    template <size_t size>
+    const char *                tagcpy(char (&out) [ size ], const std::string & in) const;
     /**
      * @brief Process the metadata in the FFmpeg file.
      * This should be called at the beginning, before reading audio data.
      * The set_text_tag() and set_picture_tag() methods of the given Encoder will
      * be used to set the metadata, with results going into the given Buffer.
      * This function will also read the actual PCM stream parameters.
-     * @param metadata_out[in] - Dictionary of output file. Metadata will be copied into it.
-     * @param metadata_in[in] - Dictionary of input file. Metadata will be copied out of it.
+     * @param[in] metadata_out - Dictionary of output file. Metadata will be copied into it.
+     * @param[in] metadata_in - Dictionary of input file. Metadata will be copied out of it.
+     * @param[in] contentstream - True if this is a content stream, i.e, audio or video. False for album arts or sub titles.
      */
-    void                        copy_metadata(AVDictionary **metadata_out, const AVDictionary *metadata_in);
+    void                        copy_metadata(AVDictionary **metadata_out, const AVDictionary *metadata_in, bool contentstream = true);
     /**
      * @brief Copy metadata from source to target
      * @return On success returns 0; on error negative AVERROR.
@@ -446,10 +505,10 @@ protected:
     /**
      * @brief Store packet in output stream.
      * @param[in] pkt - Packet to store.
-     * @param[in] type - Typo of packet: audio, video, image
+     * @param[in] mediatype - Typo of packet: audio, video, image (attachment)
      * @return On success returns 0; on error negative AVERROR.
      */
-    int                         store_packet(AVPacket *pkt, const char *type);
+    int                         store_packet(AVPacket *pkt, AVMediaType mediatype);
     /**
      * @brief Decode one audio frame
      * @param[in] pkt - Packet to decode.
@@ -522,7 +581,7 @@ protected:
      * @param[in] frame_size - Size of new frame.
      * @return On success returns 0; on error negative AVERROR.
      */
-    int                         init_audio_output_frame(AVFrame **frame, int frame_size);
+    int                         init_audio_output_frame(AVFrame **frame, int frame_size) const;
     /**
      * @brief Allocate memory for one picture.
      * @param[in] pix_fmt - Pixel format
@@ -548,7 +607,13 @@ protected:
     int                         decode(AVCodecContext *avctx, AVFrame *frame, int *got_frame, const AVPacket *pkt) const;
 #endif
     /**
-     * @brief Encode one frame worth of audio to the output file.
+     * @brief Load one audio frame from the FIFO buffer and store in frame buffer.
+     * @param[in] frame_size - Size of frame.
+     * @return On success returns 0. On error, returns a negative AVERROR value.
+     */
+    int                         create_audio_frame(int frame_size);
+    /**
+     * @brief Create one frame worth of audio to the output file.
      * @param[in] frame - Audio frame to encode
      * @param[in] data_present - 1 if frame contained data that could be encoded, 0 if not.
      * @return On success returns 0. On error, returns a negative AVERROR value.
@@ -568,12 +633,6 @@ protected:
      * @return On success returns 0. On error, returns a negative AVERROR value.
      */
     int                         encode_image_frame(const AVFrame *frame, int *data_present);
-    /**
-     * @brief Load one audio frame from the FIFO buffer, encode and write it to the output file.
-     * @param[in] frame_size - Size of frame.
-     * @return On success returns 0. On error, returns a negative AVERROR value.
-     */
-    int                         load_encode_and_write(int frame_size);
     /**
      * @brief Write the trailer of the output file container.
      * @return On success returns 0. On error, returns a negative AVERROR value.
@@ -700,7 +759,27 @@ protected:
      */
     int 						init_rescaler(AVPixelFormat in_pix_fmt, int in_width, int in_height, AVPixelFormat out_pix_fmt, int out_width, int out_height);
     /**
-     * @brief Purge FIFO buffers and report lost packet.
+     * @brief Purge all samples in audio FIFO
+     * @return Number of samples that have been purged. Function never fails.
+     */
+    int                         purge_audio_fifo();
+    /**
+     * @brief Purge all frames in audio FIFO
+     * @return Number of frames that have been purged. Function never fails.
+     */
+    size_t                      purge_audio_frame_fifo();
+    /**
+     * @brief Purge all frames in video FIFO
+     * @return Number of frames that have been purged. Function never fails.
+     */
+    size_t                      purge_video_frame_fifo();
+    /**
+     * @brief Purge all packets in HLS FIFO buffer
+     * @return Number of Packets that have been purged. Function never fails.
+     */
+    size_t                      purge_hls_fifo();
+    /**
+     * @brief Purge FIFO buffers and report lost packets/frames/samples.
      */
     void                        purge_fifos();
     /**
@@ -724,13 +803,201 @@ protected:
      * @param [out] in_pix_fmt - Input pixel format.
      * @param [out] out_pix_fmt - Output pixel format.
      */
-    void  get_pix_formats(AVPixelFormat *in_pix_fmt, AVPixelFormat *out_pix_fmt, AVCodecContext* output_codec_ctx = nullptr) const;
+    void                        get_pix_formats(AVPixelFormat *in_pix_fmt, AVPixelFormat *out_pix_fmt, AVCodecContext* output_codec_ctx = nullptr) const;
+
+    // Hardware de/encoding
+    /**
+     * Callback to negotiate the pixelFormat
+     * @param[in] input_codec_ctx - Input codec context
+     * @param[in] pix_fmts is the list of formats which are supported by the codec,
+     * it is terminated by -1 as 0 is a valid format, the formats are ordered by quality.
+     * The first is always the native one.
+     * @note The callback may be called again immediately if initialization for
+     * the selected (hardware-accelerated) pixel format failed.
+     * @warning Behavior is undefined if the callback returns a value not
+     * in the fmt list of formats.
+     * @return the chosen format
+     * - encoding: unused
+     * - decoding: Set by user, if not set the native format will be chosen.
+     */
+    static enum AVPixelFormat   get_format_static(AVCodecContext *input_codec_ctx, const enum AVPixelFormat *pix_fmts);
+    /**
+     * Callback to negotiate the pixelFormat
+     * @param[in] input_codec_ctx - Input codec context
+     * @param[in] pix_fmts is the list of formats which are supported by the codec,
+     * it is terminated by -1 as 0 is a valid format, the formats are ordered by quality.
+     * The first is always the native one.
+     * @note The callback may be called again immediately if initialization for
+     * the selected (hardware-accelerated) pixel format failed.
+     * @warning Behavior is undefined if the callback returns a value not
+     * in the fmt list of formats.
+     * @return the chosen format
+     * - encoding: unused
+     * - decoding: Set by user, if not set the native format will be chosen.
+     */
+    enum AVPixelFormat          get_format(AVCodecContext *input_codec_ctx, const enum AVPixelFormat *pix_fmts) const;
+    /**
+     * Open a device of the specified type and create an AVHWDeviceContext for it.
+     *
+     * This is a convenience function intended to cover the simple cases. Callers
+     * who need to fine-tune device creation/management should open the device
+     * manually and then wrap it in an AVHWDeviceContext using
+     * av_hwdevice_ctx_alloc()/av_hwdevice_ctx_init().
+     *
+     * The returned context is already initialized and ready for use, the caller
+     * should not call av_hwdevice_ctx_init() on it. The user_opaque/free fields of
+     * the created AVHWDeviceContext are set by this function and should not be
+     * touched by the caller.
+     *
+     * @param[out] hwaccel_enc_device_ctx - On success, a
+     * reference to the newly-created device context will be
+     * written here.
+     * @param[in] dev_type - The type of the device to create.
+     * @param[in] device - A type-specific string identifying the device to open.
+     *
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    int                         hwdevice_ctx_create(AVBufferRef **hwaccel_enc_device_ctx, AVHWDeviceType dev_type, const std::string & device) const;
+    /**
+     * @brief Add reference to hardware device context.
+     * @param[in] input_codec_ctx - Input codec context
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    int                         hwdevice_ctx_add_ref(AVCodecContext *input_codec_ctx);
+    /**
+     * @brief Free (remove reference) to hardware device context
+     * @param[inout] hwaccel_device_ctx - Hardware device context to free
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    void                        hwdevice_ctx_free(AVBufferRef **hwaccel_device_ctx);
+    /**
+     * @brief Adds a reference to an existing decoder hardware frame context or
+     * allocates a new AVHWFramesContext tied to the given hardware device context
+     * if if the decoder runs in software.
+     * @param[in] output_codec_ctx - Encoder codexc context
+     * @param[in] input_codec_ctx - Decoder codexc context
+     * @param[in] hw_device_ctx - Existing hardware device context
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    int                         hwframe_ctx_set(AVCodecContext *output_codec_ctx, AVCodecContext *input_codec_ctx, AVBufferRef *hw_device_ctx) const;
+    /**
+     * Copy data hardware surface to software.
+     * @param[in] output_codec_ctx - Codec context
+     * @param[out] sw_frame - AVFrame to copy data to
+     * @param[in] hw_frame - AVFrame to copy data from
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    int                         hwframe_copy_from_hw(AVCodecContext *output_codec_ctx, AVFrame ** sw_frame, const AVFrame *hw_frame) const;
+    /**
+     * Copy data software to a hardware surface.
+     * @param[in] output_codec_ctx - Codec context
+     * @param[out] hw_frame - AVFrame to copy data to
+     * @param[in] sw_frame - AVFrame to copy data from
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    int                         hwframe_copy_to_hw(AVCodecContext *output_codec_ctx, AVFrame ** hw_frame, const AVFrame *sw_frame) const;
+    /**
+     * @brief Get the hardware codec name as string. This is required, because e.g.
+     * the name for the software codec is libx264, but for hardware it is h264_vaapi
+     * under VAAPI.
+     * @param[in] codec_id - Id of encoder/decoder codec
+     * @param[out] codec_name - Returns the name of the codec, may be nullptr if not requitred.
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    int                         get_hw_decoder_name(AVCodecID codec_id, std::string *codec_name = nullptr) const;
+    /**
+     * @brief Get the hardware codec name as string. This is required, because e.g.
+     * the name for the software codec is libx264, but for hardware it is h264_vaapi
+     * under VAAPI.
+     * @param[in] codec_id - Id of encoder/decoder codec
+     * @param[out] codec_name - Returns the name of the codec, may be nullptr if not requitred.
+     * @return 0 on success, AVERROR_DECODER_NOT_FOUND if no codec available.
+     */
+    int                         get_hw_encoder_name(AVCodecID codec_id, std::string *codec_name = nullptr) const;
+    /**
+     * @brief Determine VAAPI codec name
+     * @param[in] codec_id - Id of encoder/decoder codec
+     * @param[out] codec_name - Name of the codec.
+     * @return 0 on success, AVERROR_DECODER_NOT_FOUND if no codec available.
+     */
+    int                         get_hw_vaapi_codec_name(AVCodecID codec_id, std::string *codec_name) const;
+    /**
+     * @brief Determine MMAL decoder codec name
+     * @param[in] codec_id - Id of encoder/decoder codec
+     * @param[out] codec_name - Name of the codec.
+     * @return 0 on success, AVERROR_DECODER_NOT_FOUND if no codec available.
+     */
+    int                         get_hw_mmal_decoder_name(AVCodecID codec_id, std::string *codec_name) const;
+    /*
+     * @brief Determine video for linux decoder codec name
+     * @param[in] codec_id - Id of encoder/decoder codec
+     * @param[out] codec_name - Name of the codec.
+     * @return 0 on success, AVERROR_DECODER_NOT_FOUND if no codec available.
+     */
+    //int                         get_hw_v4l2m2m_decoder_name(AVCodecID codec_id, std::string *codec_name) const;
+    /**
+     * @brief Determine OMX encoder codec name
+     * @param[in] codec_id - Id of encoder/decoder codec
+     * @param[out] codec_name - Name of the codec.
+     * @return 0 on success, AVERROR_DECODER_NOT_FOUND if no codec available.
+     */
+    int                         get_hw_omx_encoder_name(AVCodecID codec_id, std::string *codec_name) const;
+    /**
+     * @brief Determine video for linux encoder codec name
+     * @param[in] codec_id - Id of encoder/decoder codec
+     * @param[out] codec_name - Name of the codec.
+     * @return 0 on success, AVERROR_DECODER_NOT_FOUND if no codec available.
+     */
+    int                         get_hw_v4l2m2m_encoder_name(AVCodecID codec_id, std::string *codec_name) const;
+    /**
+     * @brief Get the software pixel format for the given hardware acceleration.
+     * @param[in] type - Selected hardware acceleration.
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    static AVPixelFormat        find_sw_fmt_by_hw_type(AVHWDeviceType type);
+    /**
+     * @brief Calculate next HLS segment from position
+     * @param[in] pos - Current transcoder position in AV_TIMEBASE fractional seconds.
+     * @return Number of next segment
+     */
+    uint32_t                    get_next_segment(int64_t pos) const;
+    /**
+     * @brief Check if segment number is next designated segment.
+     * @param[in] next_segment - Number next current segment
+     * @return Returns true if next segment should start, false if not.
+     */
+    bool                        goto_next_segment(uint32_t next_segment) const;
+
+    /**
+     * @brief Create a fake WAV header
+     * Create a fake WAV header. Inserts predicted file sizes to allow playback
+     * to start directly.
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    int                         create_fake_wav_header() const;
+    /**
+     * @brief Create a fake AIFF header
+     * Create a fake AIFF header. Inserts predicted file sizes to allow playback
+     * to start directly.
+     * @return 0 on success, a negative AVERROR code on failure.
+     */
+    int                         create_fake_aiff_header() const;
+    /**
+     * @brief Read AIFF chunk
+     * @param[in] buffer - Cache buffer to read from
+     * @param[inout] buffoffset - Byte offset into buffer. Upon return holds offset to the position of the chunk.
+     * @param[in] ID - Chunk ID (fourCC)
+     * @param[out] chunk - Buffer for chunk
+     * @param[inout] size - Size of chunk. Buffer for chunk must be large enough to hold it. Upon return holds the actual size of the chunk read.
+     * @return Returns 0 if successful or -1 on error or end of file. Check buffer->eof().
+     */
+    int                         read_aiff_chunk(Buffer *buffer, size_t *buffoffset, const char *ID, uint8_t *chunk, size_t *size) const;
 
 private:
     FileIO *                    m_fileio;                   /**< @brief FileIO object of input file */
     bool                        m_close_fileio;             /**< @brief If we own the FileIO object, we may close it in the end. */
     time_t                      m_mtime;                    /**< @brief Modified time of input file */
-    std::recursive_mutex        m_mutex;                    /**< @brief Access mutex */
+    std::recursive_mutex        m_seek_to_fifo_mutex;       /**< @brief Access mutex for seek FIFO */
     std::queue<uint32_t>        m_seek_to_fifo;             /**< @brief Stack of seek requests. Will be processed FIFO */
     volatile uint32_t           m_last_seek_frame_no;       /**< @brief If not 0, this is the last frame that we seeked to. Video sources only. */
     bool                        m_have_seeked;              /**< @brief After seek operations this is set to make sure the trancoding result is marked RESULTCODE_INCOMPLETE to start transcoding over next access to fill the gaps. */
@@ -747,13 +1014,14 @@ private:
     AVAudioResampleContext *    m_audio_resample_ctx;       /**< @brief AVResample context for audio resampling */
 #endif
     AVAudioFifo *               m_audio_fifo;               /**< @brief Audio sample FIFO */
+    std::queue<AVFrame*>        m_audio_frame_fifo;         /**< @brief Audio frame FIFO */
 
     // Video conversion and buffering
     SwsContext *                m_sws_ctx;                  /**< @brief Context for video filtering */
     AVFilterContext *           m_buffer_sink_context;      /**< @brief Video filter sink context */
     AVFilterContext *           m_buffer_source_context;    /**< @brief Video filter source context */
     AVFilterGraph *             m_filter_graph;             /**< @brief Video filter graph */
-    std::queue<AVFrame*>        m_video_fifo;               /**< @brief Video frame FIFO */
+    std::queue<AVFrame*>        m_video_frame_fifo;         /**< @brief Video frame FIFO */
     int64_t                     m_pts;                      /**< @brief Generated PTS */
     int64_t                     m_pos;                      /**< @brief Generated position */
 
@@ -761,6 +1029,7 @@ private:
     OUTPUTFILE                  m_out;                      /**< @brief Output file information */
 
     uint32_t                    m_current_segment;          /**< @brief HLS only: Segment file number currently being encoded */
+    bool                        m_insert_keyframe;          /**< @brief HLS only: Allow insertion of 1 keyframe */
 
     // If the audio and/or video stream is copied, packets will be stuffed into the packet queue.
     bool                        m_copy_audio;               /**< @brief If true, copy audio stream from source to target (just remux, no recode). */
@@ -770,10 +1039,28 @@ private:
 
     Buffer *                    m_buffer;                   /**< @brief Pointer to cache buffer object */
 
-    bool                        m_reset_pts;                /**< @brief We have to reset audio/video pts to the new position */
+    uint32_t                    m_reset_pts;                /**< @brief We have to reset audio/video pts to the new position */
     uint32_t                    m_fake_frame_no;            /**< @brief The MJEPG codec requires monotonically growing PTS values so we fake some to avoid them going backwards after seeks */
 
     static const PRORES_BITRATE m_prores_bitrate[];         /**< @brief ProRes bitrate table. Used for file size prediction. */
+
+    // Hardware acceleration
+    static const DEVICETYPE_MAP m_devicetype_map;               /**< @brief List of AVPixelFormats mapped to hardware acceleration types */
+    HWACCELMODE                 m_hwaccel_enc_mode;             /**< @brief Current hardware acceleration mode for encoder */
+    HWACCELMODE                 m_hwaccel_dec_mode;             /**< @brief Current hardware acceleration mode for decoder */
+    bool						m_hwaccel_enable_enc_buffering; /**< @brief Enable hardware acceleration frame buffers for encoder */
+    bool                        m_hwaccel_enable_dec_buffering; /**< @brief Enable hardware acceleration frame buffers for decoder */
+    AVBufferRef *               m_hwaccel_enc_device_ctx;       /**< @brief Hardware acceleration device context for encoder */
+    AVBufferRef *               m_hwaccel_dec_device_ctx;       /**< @brief Hardware acceleration device context for decoder */
+    AVPixelFormat               m_enc_hw_pix_fmt;               /**< @brief Requested encoder hardware pixel format */
+    AVPixelFormat               m_dec_hw_pix_fmt;               /**< @brief Requested decoder hardware pixel format */
+
+#define FFMPEGFS_AUDIO      static_cast<uint32_t>(0x0001)   /**< @brief Denote an audio stream */
+#define FFMPEGFS_VIDEO      static_cast<uint32_t>(0x0002)   /**< @brief Denote a video stream */
+
+    uint32_t                    m_active_stream_msk;        /**< @brief HLS: Currently active streams bit mask. Set FFMPEGFS_AUDIO and/or FFMPEGFS_VIDEO */
+    uint32_t                    m_inhibit_stream_msk;       /**< @brief HLS: Currently inhibited streams bit mask. Packets temporarly go to m_hls_packet_fifo and will be prepended to next segment. Set FFMPEGFS_AUDIO and/or FFMPEGFS_VIDEO */
+    std::queue<AVPacket*>       m_hls_packet_fifo;          /**< @brief HLS packet FIFO */
 };
 
 #endif // FFMPEG_TRANSCODER_H
