@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
  */
 
 /**
- * @file
+ * @file ffmpeg_utils.h
  * @brief Various FFmpegfs utility functions
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef FFMPEG_UTILS_H
@@ -52,6 +52,18 @@
 #include <vector>
 #include <regex>
 
+// Support for libavresample has finally been removed from FFmpeg API
+// as of 14.04.2021, see this commit:
+//
+// commit 420cedd49745b284c35d97b936b71ff79b43bdf7
+// Author: Andreas Rheinhardt <andreas.rheinhardt@outlook.com>
+// Date:   Wed Apr 14 04:33:24 2021 +0200
+//
+//     libavresample: Remove deprecated library
+//
+//     Deprecated in c29038f3041a4080342b2e333c1967d136749c0f.
+//     The resample filter based upon this library has been removed as well.
+
 #if !defined(USE_LIBSWRESAMPLE) && !defined(USE_LIBAVRESAMPLE)
 #error "Must have either libswresample (preferred choice for FFMpeg) or libavresample (with libav)."
 #endif
@@ -91,12 +103,12 @@ extern "C" {
 /**
  * Add av_get_media_type_string function if missing
  */
-#define HAVE_MEDIA_TYPE_STRING              (LIBAVUTIL_VERSION_MICRO >= 100 && LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(55, 34, 101))
+#define HAVE_MEDIA_TYPE_STRING                  (LIBAVUTIL_VERSION_MICRO >= 100 && LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(55, 34, 101))
 #if HAVE_MEDIA_TYPE_STRING
 /**
  *  Map to av_get_media_type_string function.
  */
-#define get_media_type_string               av_get_media_type_string
+#define get_media_type_string                   av_get_media_type_string
 #else
 /**
  * @brief av_get_media_type_string is missing, so we provide our own.
@@ -106,6 +118,14 @@ extern "C" {
 const char *get_media_type_string(enum AVMediaType media_type);
 #endif
 
+/**
+ * Min. FFmpeg version for VULKAN hardware acceleration support
+ * 2020-02-04 - xxxxxxxxxx - lavu 56.39.100 - hwcontext.h
+ *   Add AV_PIX_FMT_VULKAN
+ *   Add AV_HWDEVICE_TYPE_VULKAN and implementation.
+ */
+#define HAVE_VULKAN_HWACCEL                     (LIBAVUTIL_VERSION_MICRO >= 100 && LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(56, 39, 100))
+
 #ifndef AV_ROUND_PASS_MINMAX
 /**
  *  Ignore if this is missing
@@ -115,27 +135,27 @@ const char *get_media_type_string(enum AVMediaType media_type);
 
 // These once had a different name
 #if !defined(AV_CODEC_CAP_DELAY) && defined(CODEC_CAP_DELAY)
-#define AV_CODEC_CAP_DELAY              	CODEC_CAP_DELAY                         /**< @brief AV_CODEC_CAP_DELAY is missing in older FFmpeg versions */
+#define AV_CODEC_CAP_DELAY                      CODEC_CAP_DELAY                         /**< @brief AV_CODEC_CAP_DELAY is missing in older FFmpeg versions */
 #endif
 
 #if !defined(AV_CODEC_CAP_TRUNCATED) && defined(CODEC_CAP_TRUNCATED)
-#define AV_CODEC_CAP_TRUNCATED          	CODEC_CAP_TRUNCATED                     /**< @brief AV_CODEC_CAP_TRUNCATED is missing in older FFmpeg versions */
+#define AV_CODEC_CAP_TRUNCATED                  CODEC_CAP_TRUNCATED                     /**< @brief AV_CODEC_CAP_TRUNCATED is missing in older FFmpeg versions */
 #endif
 
 #if !defined(AV_CODEC_FLAG_TRUNCATED) && defined(CODEC_FLAG_TRUNCATED)
-#define AV_CODEC_FLAG_TRUNCATED         	CODEC_FLAG_TRUNCATED                    /**< @brief AV_CODEC_FLAG_TRUNCATED is missing in older FFmpeg versions */
+#define AV_CODEC_FLAG_TRUNCATED                 CODEC_FLAG_TRUNCATED                    /**< @brief AV_CODEC_FLAG_TRUNCATED is missing in older FFmpeg versions */
 #endif
 
 #ifndef AV_CODEC_FLAG_GLOBAL_HEADER
-#define AV_CODEC_FLAG_GLOBAL_HEADER     	CODEC_FLAG_GLOBAL_HEADER                /**< @brief AV_CODEC_FLAG_GLOBAL_HEADER is missing in older FFmpeg versions */
+#define AV_CODEC_FLAG_GLOBAL_HEADER             CODEC_FLAG_GLOBAL_HEADER                /**< @brief AV_CODEC_FLAG_GLOBAL_HEADER is missing in older FFmpeg versions */
 #endif
 
 #ifndef FF_INPUT_BUFFER_PADDING_SIZE
-#define FF_INPUT_BUFFER_PADDING_SIZE    	256                                     /**< @brief FF_INPUT_BUFFER_PADDING_SIZE is missing in newer FFmpeg versions */
+#define FF_INPUT_BUFFER_PADDING_SIZE            256                                     /**< @brief FF_INPUT_BUFFER_PADDING_SIZE is missing in newer FFmpeg versions */
 #endif
 
 #ifndef AV_CODEC_CAP_VARIABLE_FRAME_SIZE
-#define AV_CODEC_CAP_VARIABLE_FRAME_SIZE	CODEC_CAP_VARIABLE_FRAME_SIZE           /**< @brief AV_CODEC_CAP_VARIABLE_FRAME_SIZE is missing in older FFmpeg versions */
+#define AV_CODEC_CAP_VARIABLE_FRAME_SIZE        CODEC_CAP_VARIABLE_FRAME_SIZE           /**< @brief AV_CODEC_CAP_VARIABLE_FRAME_SIZE is missing in older FFmpeg versions */
 #endif
 
 #if (LIBAVUTIL_VERSION_MAJOR > 54)
@@ -173,7 +193,9 @@ typedef enum FILETYPE
     FILETYPE_JPG,
     FILETYPE_BMP,
     FILETYPE_TS,
-    FILETYPE_HLS
+    FILETYPE_HLS,
+    FILETYPE_FLAC,
+    FILETYPE_MKV,
 } FILETYPE;
 
 /**
@@ -237,29 +259,159 @@ typedef enum AUTOCOPY
   */
 typedef enum RECODESAME
 {
-     RECODESAME_NO = 0,     /**< @brief Never recode to same format. */
-     RECODESAME_YES,        /**< @brief Always recode to same format. */
+    RECODESAME_NO = 0,     /**< @brief Never recode to same format. */
+    RECODESAME_YES,        /**< @brief Always recode to same format. */
 } RECODESAME;
 
+/**
+  * List of sample formats.
+  * User selection, we don't care about planar or interleaved.
+  */
+typedef enum SAMPLE_FMT
+{
+    SAMPLE_FMT_DONTCARE = -1,   /**< @brief Don't care, leave to FFmpegfs to choose */
+    SAMPLE_FMT_8,               /**< @brief 8 bit integer */
+    SAMPLE_FMT_16,              /**< @brief 16 bit integer */
+    SAMPLE_FMT_24,              /**< @brief 24 bit integer */
+    SAMPLE_FMT_32,              /**< @brief 32 bit integer */
+    SAMPLE_FMT_64,              /**< @brief 64 bit integer */
+    SAMPLE_FMT_F16,             /**< @brief 16 bit floating point */
+    SAMPLE_FMT_F24,             /**< @brief 24 bit floating point */
+    SAMPLE_FMT_F32,             /**< @brief 32 bit floating point */
+    SAMPLE_FMT_F64              /**< @brief 64 bit floating point */
+} SAMPLE_FMT;
+
+/**
+ * Format options: Defines file extension, codecs etc.
+ * for each format.
+ */
+struct Format_Options
+{
+    friend class FFmpegfs_Format;
+
+    typedef std::vector<AVCodecID>  CODEC_VECT;                /**< @brief Vector with valid codec ids for file format */
+
+    /**
+      * Format options: Audio/video codecs and sample format
+      */
+    typedef struct _tagFORMAT
+    {
+        CODEC_VECT      m_video_codec;      /**< @brief AVCodec used for video encoding */
+        CODEC_VECT      m_audio_codec;      /**< @brief AVCodec used for audio encoding */
+        AVSampleFormat  m_sample_format;    /**< @brief AVSampleFormat for audio encoding, may be AV_SAMPLE_FMT_NONE for "don't care" */
+    } FORMAT;
+
+    typedef std::map<SAMPLE_FMT, const FORMAT> FORMAT_MAP;   /**< @brief Map of formats. One entry per format derivative. */
+
+public:
+    /**
+     * @brief Construct Format_Options object with defaults (empty)
+     */
+    Format_Options();
+
+    /**
+     * @brief Construct Format_Options object
+     * @param[in] format_name - Descriptive name of the format, e.g. "Opus Audio",
+     * @param[in] fileext - File extension: mp4, mp3, flac or other
+     * @param[in] format - Format options: Possible audio/video codecs and sample formats
+     * @param[in] albumart_supported - true if album arts are supported (eg. mp3) or false if not (e.g. wav, aiff
+     */
+    Format_Options(
+            const std::string & format_name,
+            const std::string & fileext,
+            const FORMAT_MAP &  format,
+            bool                albumart_supported
+            );
+
+    /**
+     * @brief Convert destination type to "real" type, i.e., the file extension to be used.
+     * @note Currently "prores" is mapped to "mov".
+     * @return Destination type
+     */
+    const std::string & format_name() const;
+
+    /**
+     * @brief Get file extension
+     * @return File extension
+     */
+    const std::string & fileext() const;
+
+    /**
+     * @brief Get video codec_id
+     * @return Returns video codec_id
+     */
+    AVCodecID           video_codec() const;
+    /**
+     * @brief Check if video codec/file format combination is supported
+     * @param codec_id - Codec ID to check
+     * @return Returns true if supported, false if not.
+     */
+    bool                is_video_codec_supported(AVCodecID codec_id) const;
+    /**
+     * @brief Create a list of supported audio codecs for current audio codec
+     * @return Returns comma separated list of formats, or empty if not available.
+     */
+    std::string         video_codec_list() const;
+
+    /**
+     * @brief Get audio codec_id
+     * @return Returns audio codec_id
+     */
+    AVCodecID           audio_codec() const;
+    /**
+     * @brief Check if audio codec/file format combination is supported
+     * @param codec_id - Codec ID to check
+     * @return Returns true if supported, false if not.
+     */
+    bool                is_audio_codec_supported(AVCodecID codec_id) const;
+    /**
+     * @brief Create a list of supported audio codecs for current audio codec
+     * @return Returns comma separated list of formats, or empty if not available.
+     */
+    std::string         audio_codec_list() const;
+
+    /**
+     * @brief Get sample format (bit width)
+     * @return Returns sample format
+     */
+    AVSampleFormat      sample_format() const;
+    /**
+     * @brief Check if audio codec/sample format combination is supported
+     * @return Returns true if supported, false if not.
+     */
+    bool                is_sample_fmt_supported() const;
+    /**
+     * @brief Create a list of supported sample formats for current audio codec
+     * @return Returns comma separated list of formats, or empty if not available.
+     */
+    std::string         sample_fmt_list() const;
+
+protected:
+    /**
+     *  @brief Descriptive name of the format.
+     *  Descriptive name of the format, e.g. "opus", "mpegts".
+     *  Please note that m_format_name is used to select the FFmpeg container
+     *  by passing it to avformat_alloc_output_context2().
+     *  Mostly, but not always, same as m_fileext.
+     */
+    std::string     m_format_name;
+    std::string     m_fileext;              /**< @brief File extension: mp4, mp3, flac or other. Mostly, but not always, same as m_format_name. */
+    FORMAT_MAP      m_format_map;           /**< @brief Format definition (audio/videocodec, sample format) */
+    bool            m_albumart_supported;   /**< @brief true if album arts are supported (eg. mp3) or false if not (e.g. wav, aiff) */
+};
+
 /**
  * @brief The #FFmpegfs_Format class
  */
 class FFmpegfs_Format
 {
+    typedef std::map<FILETYPE, const Format_Options> OPTIONS_MAP;   /**< @brief Map of options. One entry per supported destination type. */
+
 public:
     /**
      * @brief Construct FFmpegfs_Format object
      */
     FFmpegfs_Format();
-    /**
-     * @brief Construct FFmpegfs_Format object
-     * @param[in] format_name - Name of this format, e.g. "MP4"
-     * @param[in] fileext - File extension for this format, e.g. "mp4"
-     * @param[in] filetype - File type, MP3, MP4, OPUS etc.
-     * @param[in] video_codec_id - AVCodec used for video encoding
-     * @param[in] audio_codec_id - AVCodec used for audio encoding
-     */
-    FFmpegfs_Format(const std::string & format_name, const std::string &fileext, FILETYPE filetype, AVCodecID video_codec_id, AVCodecID audio_codec_id);
 
     /**
      * @brief Get codecs for the selected destination type.
@@ -288,16 +440,57 @@ public:
      * @return Returns selected filetype.
      */
     FILETYPE            filetype() const;
+
     /**
      * @brief Get video codec_id
      * @return Returns video codec_id
      */
-    AVCodecID           video_codec_id() const;
+    AVCodecID           video_codec() const;
+    /**
+     * @brief Check if video codec/file format combination is supported
+     * @param codec_id - Codec ID to check
+     * @return Returns true if supported, false if not.
+     */
+    bool                is_video_codec_supported(AVCodecID codec_id) const;
+    /**
+     * @brief Create a list of supported audio codecs for current audio codec
+     * @return Returns comma separated list of formats, or empty if not available.
+     */
+    std::string         video_codec_list() const;
+
     /**
      * @brief Get audio codec_id
      * @return Returns audio codec_id
      */
-    AVCodecID           audio_codec_id() const;
+    AVCodecID           audio_codec() const;
+    /**
+     * @brief Check if audio codec/file format combination is supported
+     * @param codec_id - Codec ID to check
+     * @return Returns true if supported, false if not.
+     */
+    bool                is_audio_codec_supported(AVCodecID codec_id) const;
+    /**
+     * @brief Create a list of supported audio codecs for current audio codec
+     * @return Returns comma separated list of formats, or empty if not available.
+     */
+    std::string         audio_codec_list() const;
+
+    /**
+     * @brief Get sample format (bit width)
+     * @return Returns sample format
+     */
+    AVSampleFormat      sample_format() const;
+    /**
+     * @brief Check if audio codec/sample format combination is supported
+     * @return Returns true if supported, false if not.
+     */
+    bool                is_sample_fmt_supported() const;
+    /**
+     * @brief Create a list of supported sample formats for current audio codec
+     * @return Returns comma separated list of formats, or empty if not available.
+     */
+    std::string         sample_fmt_list() const;
+
     /**
      * @brief Check if this si some sort of multi file format
      * (any of the following: is_frameset() or is_hls()).
@@ -314,14 +507,18 @@ public:
      * @return Returns true for formats that create an HLS set including the m3u file.
      */
     bool                is_hls() const;
+    /**
+     * @brief Check if album arts are supported
+     * @return true if album arts are supported or false if not
+     */
+    bool                albumart_supported() const;
 
 protected:
-    std::string m_format_name;              /**< @brief Descriptive name of the format, e.g. "Opus Audio". */
-    std::string m_desttype;                 /**< @brief Destination type: mp4, mp3 or other */
-    std::string m_fileext;                  /**< @brief File extension: mp4, mp3, flac or other */
-    FILETYPE    m_filetype;                 /**< @brief File type, MP3, MP4, OPUS etc. */
-    AVCodecID   m_video_codec_id;           /**< @brief AVCodec used for video encoding */
-    AVCodecID   m_audio_codec_id;           /**< @brief AVCodec used for audio encoding */
+    const Format_Options        m_empty_options;    /**< @brief Set of empty (invalid) options as default */
+    const Format_Options *      m_cur_opts;         /**< @brief Currently selected options. Will never be nullptr */
+    static const OPTIONS_MAP    m_options_map;      /**< @brief Map of options. One entry per supported destination type. */
+    std::string                 m_desttype;         /**< @brief Destination type: mp4, mp3 or other */
+    FILETYPE                    m_filetype;         /**< @brief File type, MP3, MP4, OPUS etc. */
 };
 
 /**
@@ -361,6 +558,13 @@ const std::string & remove_path(std::string *filepath);
  * @return Returns constant reference to filename.
  */
 const std::string & remove_ext(std::string *filepath);
+/**
+ * @brief Find extension in allowlist, if existing.
+ * @param[in] ext - Extension, if found.
+ * @param[in] allowlist - Allow list to inspect.
+ * @return Returns true if extension was found, false if there was none
+ */
+bool                allow_list_ext(const std::string & ext, const std::string & allowlist);
 /**
  * @brief Find extension in filename, if existing.
  * @param[in] ext - Extension, if found.
@@ -388,13 +592,6 @@ const std::string & append_ext(std::string * filepath, const std::string & ext);
  * @return Copy of the input string. Remember to delete the allocated memory.
  */
 char *              new_strdup(const std::string & str);
-/**
- * @brief Get destination filename. Replaces extension and path.
- * @param[in] destfilepath - Destination name and path.
- * @param[in] filepath - Source filename and path.
- * @return Returns constant reference to destfilepath.
- */
-const std::string & get_destname(std::string *destfilepath, const std::string & filepath);
 /**
  * @brief Get FFmpeg error string for errnum. Internally calls av_strerror().
  * @param[in] errnum - FFmpeg error code.
@@ -553,6 +750,12 @@ int                 supports_albumart(FILETYPE filetype);
  * @return On success returns FILETYPE enum; on error returns FILETYPE_UNKNOWN.
  */
 FILETYPE            get_filetype(const std::string & desttype);
+/**
+* @brief Convert FILETYPE enum to human readable text.
+ * @param[in] filetype - FILETYPE enum value to convert.
+ * @return FILETYPE enum as text or "INVALID" if not known.
+ */
+std::string         get_filetype_text(FILETYPE filetype);
 /**
  * @brief Get the FFmpegfs filetype, desttypelist must be a comma separated list of FFmpeg's "official" short names for formats.
  * Will return the first match. Same as get_filetype, but accepts a comma separated list.
@@ -566,6 +769,15 @@ FILETYPE            get_filetype_from_list(const std::string & desttypelist);
  * @return On success returns 0; on error negative AVERROR.
  */
 int                 print_stream_info(const AVStream* stream);
+/**
+ * Fill the provided buffer with a string containing a FourCC (four-character
+ * code) representation.
+ *
+ * @param buf - Upon return, filled in with the FourCC representation.
+ * @param fourcc - The fourcc to represent
+ * @return The buffer in input.
+ */
+std::string         fourcc_make_string(std::string * buf, uint32_t fourcc);
 /**
  * @brief Compare value with pattern.
  * @param[in] value - Value to check.
@@ -648,7 +860,7 @@ bool                is_album_art(AVCodecID codec_id, const AVRational *frame_rat
  * @param[in] rhs - right hand string
  * @return -1 if lhs < rhs; 0 if lhs == rhs and 1 if lhs > rhs
  */
-bool                nocasecompare(const std::string & lhs, const std::string &rhs);
+int                 nocasecompare(const std::string & lhs, const std::string &rhs);
 
 /**
  * @brief The comp struct to make std::string find operations case insensitive
@@ -659,11 +871,11 @@ struct comp
      * @brief operator () to make std::string find operations case insensitive
      * @param[in] lhs - left hand string
      * @param[in] rhs - right hand string
-     * @return -1 if lhs < rhs; 0 if lhs == rhs and 1 if lhs > rhs
+     * @return true if lhs < rhs; false if lhs >= rhs
      */
     bool operator() (const std::string& lhs, const std::string& rhs) const
     {
-        return nocasecompare(lhs, rhs);
+        return (nocasecompare(lhs, rhs) < 0);
     }
 };
 
@@ -697,11 +909,90 @@ std::string         make_filename(uint32_t file_no, const std::string &fileext);
  */
 bool                file_exists(const std::string & filename);
 
+/** Save version of hwdevice_get_type_name:
+ * Get the string name of an AVHWDeviceType.
+ *
+ * @param[in] dev_type - Type from enum AVHWDeviceType.
+ * @return Pointer to a static string containing the name, or "unknown" if the type
+ *         is not valid.
+ */
+const char *        hwdevice_get_type_name(AVHWDeviceType dev_type);
+
+/**
+  * Detected encoding types
+  */
+typedef enum ENCODING
+{
+    ENCODING_ASCII          = -1,       /**< @brief Some sort of ASCII encoding. */
+    ENCODING_UTF8_BOM       = -2,       /**< @brief UTF-8 with bottom mark. */
+    ENCODING_UTF16LE_BOM    = -3,       /**< @brief UTF-16 little-endian with bottom mark. */
+    ENCODING_UTF16BE_BOM    = -4,       /**< @brief UTF-16 big-endian with bottom mark. */
+    ENCODING_UTF32LE_BOM    = -5,       /**< @brief UTF-16 little-endian with bottom mark. */
+    ENCODING_UTF32BE_BOM    = -6,       /**< @brief UTF-16 big-endian with bottom mark. */
+} ENCODING;
+
+/**
+ * @brief Convert almost any encoding to UTF-8.
+ * To get a list of all possible encodings run "iconv --list".
+ * @param[in] text - Text to be converted
+ * @param[in] encoding - Encoding of input text.
+ * @return Returns 0 if successful and the converted text,
+ * or errno value on error and text is unchanged.
+ */
+int                 to_utf8(std::string & text, const std::string & encoding);
+/**
+ * @brief Try to detect the encoding of str. This is relatively realiable,
+ * but may be wrong.
+ * @param[in] str - Text string to be checked.
+ * @param[out] encoding - Detected encoding.
+ * @return Returns 0 if successful, or CHARDET_OUT_OF_MEMORY/CHARDET_MEM_ALLOCATED_FAIL
+ * on error.
+ */
+int                 get_encoding (const char * str, std::string & encoding);
+/**
+ * @brief Read text file and return in UTF-8 format, no matter in which
+ * encoding the input file is. UTF-8/16/32 with BOM will always return a
+ * correct result. For all other encodings the function tries to detect it,
+ * that may fail.
+ * @param[in] path - Path and filename of input file
+ * @param[out] result - File contents as UTF-8
+ * @return Returns one of the ENCODING enum values on success,
+ * or errno on error. Basically a return code > 0 means there is an error.
+ */
+int                 read_file(const std::string & path, std::string & result);
+
 /**
  * @brief Properly fill in all size related members in stat struct
- * @param st[inout] stat structure to update
- * @param size[in] size value to copy
+ * @param[inout] st stat structure to update
+ * @param[in] size size value to copy
  */
 void                stat_set_size(struct stat *st, size_t size);
 
+/**
+ * @brief Detect if we are running under Docker.
+ * @return Returns true, if running under Docker, or false if not.
+ */
+bool                detect_docker(void);
+
+/**
+ * @brief Iterate through all elements in map and search for the passed element.
+ * @param[in] mapOfWords - map to search.
+ * @param[in] value - Search value
+ * @return If found, retuns const_iterator to element. Returns mapOfWords.cend() if not.
+ */
+template <typename T>
+typename std::map<const std::string, const T, comp>::const_iterator search_by_value(const std::map<const std::string, const T, comp> & mapOfWords, T value)
+{
+    typename std::map<const std::string, const T, comp>::const_iterator it = mapOfWords.cbegin();
+    while (it != mapOfWords.cend())
+    {
+        if (it->second == value)
+        {
+            return it;
+        }
+        ++it;
+    }
+    return mapOfWords.cend();
+}
+
 #endif
