@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 2006-2008 David Collett
  * Copyright (C) 2008-2013 K. Henriksson
- * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,14 +27,14 @@
 #pragma once
 
 /**
- * @file
+ * @file ffmpegfs.h
  * @ingroup ffmpegfs
  * @brief Main include for FFmpegfs project
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
  * @copyright Copyright (C) 2006-2008 David Collett @n
  * Copyright (C) 2008-2013 K. Henriksson @n
- * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 /** @mainpage FFmpegfs FUSE Filesystem
@@ -49,6 +49,8 @@
  * @section ffmpeg_documentation Documentation
  *
  * @li [Read me!](README.md)
+ * @li [Man Pages](manpages.html)
+ * @li [History](HISTORY.md)
  * @li [Installation instructions](INSTALL.md)
  * @li @ref todo
  * @li @ref bug
@@ -93,12 +95,57 @@
 #include "ffmpeg_utils.h"
 #include "fileio.h"
 
+/**
+  *
+  * @brief Hardware acceleration types.
+  */
+typedef enum HWACCELAPI
+{
+    HWACCELAPI_NONE,
+    HWACCELAPI_VAAPI,               /**< Intel: VAAPI */
+
+    HWACCELAPI_MMAL,                /**< Raspberry: MMAL */
+    HWACCELAPI_OMX,                 /**< Raspberry: OpenMAX */
+
+    // Additional formats
+#if 0
+    HWACCELAPI_CUDA,                /**< Nividia: CUDA    to be added */
+    HWACCELAPI_V4L2M2M,             /**< v4l2 mem to mem  to be added */
+    HWACCELAPI_VDPAU,               /**< VDPAU            to be added */
+    HWACCELAPI_QSV,                 /**< QSV              to be added */
+    HWACCELAPI_OPENCL,              /**< OPENCL           to be added */
+#if HAVE_VULKAN_HWACCEL
+    HWACCELAPI_VULKAN,              /**< VULKAN           to be added */
+#endif // HAVE_VULKAN_HWACCEL
+#if __APPLE__
+    // MacOS acceleration APIs not supported
+    HWACCELAPI_VIDEOTOOLBOX,        /**< VIDEOTOOLBOX     not supported */
+#endif
+#if __ANDROID__
+    // Android acceleration APIs not supported
+    HWACCELAPI_MEDIACODEC,          /**< MediaCodec API   not supported */
+#endif
+#if _WIN32
+// Windows acceleration APIs not supported
+    HWACCELAPI_DRM,                 /**< DRM              not supported */
+    HWACCELAPI_DXVA2,               /**< DXVA2            not supported */
+    HWACCELAPI_D3D11VA,             /**< D3D11VA          not supported */
+#endif
+#endif
+} HWACCELAPI;
+
+typedef std::multimap<AVCodecID, int> HWACCEL_BLOCKED_MAP;      /**< @brief Map command line option to AVCodecID */
+
+// Format
+extern FFmpegfs_Format     ffmpeg_format[2];                    /**< @brief Two FFmpegfs_Format infos, 0: video file, 1: audio file */
+
 /**
  * @brief Global program parameters
  */
 extern struct FFMPEGFS_PARAMS
 {
     FFMPEGFS_PARAMS();
+    ~FFMPEGFS_PARAMS();
 
     /**
      * @brief Check for smart transcode mode
@@ -127,17 +174,20 @@ extern struct FFMPEGFS_PARAMS
     // Paths
     std::string         m_basepath;                 /**< @brief Base path: Files from this directory (including all sub directories) will be mapped to m_mountpath. */
     std::string         m_mountpath;                /**< @brief Mount path: Files from m_mountpath will be mapped to this directory. */
+
     // Output type
+    AVCodecID           m_audio_codec;              /**< @brief Either AV_CODEC_ID_NONE for default, or a user selected codec */
+    AVCodecID           m_video_codec;              /**< @brief Either AV_CODEC_ID_NONE for default, or a user selected codec */
     AUTOCOPY            m_autocopy;                 /**< @brief Copy streams if codec matches */
     RECODESAME          m_recodesame;               /**< @brief Recode to same format options */
     PROFILE             m_profile;					/**< @brief Target profile: Firefox, MS Edge/IE or other */
     PRORESLEVEL         m_level;                    /**< @brief Level, currently proxy/hq/lt/HQ (ProRes only) */
-    // Format
-    FFmpegfs_Format     m_format[2];                /**< @brief Two FFmpegfs_Format infos, 0: video file, 1: audio file  */
     // Audio
     BITRATE             m_audiobitrate;             /**< @brief Output audio bit rate (bits per second) */
     int                 m_audiosamplerate;          /**< @brief Output audio sample rate (in Hz) */
     int                 m_audiochannels;            /**< @brief Max. number of audio channels */
+    SAMPLE_FMT          m_sample_fmt;               /**< @brief Sample format */
+    std::string         m_extensions;               /**< @brief Comma separated lists of supplementary file extensions */
     // Video
     BITRATE             m_videobitrate;             /**< @brief Output video bit rate (bits per second) */
     int                 m_videowidth;               /**< @brief Output video width */
@@ -145,6 +195,15 @@ extern struct FFMPEGFS_PARAMS
     int                 m_deinterlace;              /**< @brief 1: deinterlace video, 0: no deinterlace */
     // HLS Options
     int64_t             m_segment_duration;         /**< @brief Duration of one HLS segment file, in AV_TIME_BASE fractional seconds. */
+    int64_t             m_min_seek_time_diff;       /**< @brief Minimum time diff from current to next requested segment to perform a seek, in AV_TIME_BASE fractional seconds. */
+    // Hardware acceleration
+    HWACCELAPI          m_hwaccel_enc_API;          /**< @brief Encoder API */
+    AVHWDeviceType      m_hwaccel_enc_device_type;  /**< @brief Enable hardware acceleration buffering for encoder */
+    std::string         m_hwaccel_enc_device;       /**< @brief Encoder device. May be AUTO to auto detect or empty */
+    HWACCELAPI          m_hwaccel_dec_API;          /**< @brief Decoder API */
+    AVHWDeviceType      m_hwaccel_dec_device_type;  /**< @brief Enable hardware acceleration buffering for decoder */
+    std::string         m_hwaccel_dec_device;       /**< @brief Decoder device. May be AUTO to auto detect or empty */
+    HWACCEL_BLOCKED_MAP*m_hwaccel_dec_blocked;      /**< @brief List of blocked decoders and optional profiles */
     // Album arts
     int                 m_noalbumarts;              /**< @brief skip album arts */
     // Virtual script
@@ -180,16 +239,18 @@ extern struct FFMPEGFS_PARAMS
 
 class Cache_Entry;
 
+extern bool             docker_client;              /**< @brief True if running inside a Docker container */
+
 /**
  * @brief Fuse operations struct
  */
-extern fuse_operations ffmpegfs_ops;
+extern fuse_operations  ffmpegfs_ops;
 
 class thread_pool;
 /**
  * @brief Thread pool object
  */
-extern thread_pool*         tp;
+extern thread_pool*     tp;
 
 /**
  * @brief Initialise FUSE operation structure.
@@ -243,6 +304,9 @@ bool            transcoder_cache_clear(void);
  * @brief Add new virtual file to internal list.
  *
  * For Bluray/DVD/VCD actually no physical input file exists, so virtual file and origfile are the same.
+ * The input file is handled by the BlurayIO or VcdIO classes.
+ * For cue sheets, the original (huge) input file is used. Start positions are sought, end positions
+ * reported as EOF.
  *
  * @param[in] type - Type of virtual file.
  * @param[in] virtfile - Name of virtual file.
@@ -311,23 +375,79 @@ LPVIRTUALFILE   find_original(std::string *filepath);
  */
 LPVIRTUALFILE   find_parent(const std::string & origpath);
 
+/**
+ * @brief Convert SAMPLE_FMT enum to human readable text.
+ * @return SAMPLE_FMT enum as text or "INVALID" if not known.
+ */
+std::string     get_sampleformat_text(SAMPLE_FMT sample_fmt);
+/**
+ * @brief Convert AVCodecID enum for audio codec to human readable text.
+ * @param[in] audio_codec - AVCodecID enum value to convert.
+ * @return AVCodecID as text or "INVALID" if not known.
+ */
+std::string 	get_audio_codec_text(AVCodecID audio_codec);
+/**
+ * @brief Convert AVCodecID enum for video codec to human readable text.
+ * @param[in] video_codec - AVCodecID enum value to convert.
+ * @return AVCodecID as text or "INVALID" if not known.
+ */
+std::string 	get_video_codec_text(AVCodecID video_codec);
+/**
+ * @brief Convert AUTOCOPY enum to human readable text.
+ * @param[in] autocopy - AUTOCOPY enum value to convert.
+ * @return AUTOCOPY enum as text or "INVALID" if not known.
+ */
+std::string 	get_autocopy_text(AUTOCOPY autocopy);
+/**
+ * @brief Convert RECODESAME enum to human readable text.
+ * @param[in] recode - RECODESAME enum value to convert.
+ * @return RECODESAME enum as text or "INVALID" if not known.
+ */
+std::string 	get_recodesame_text(RECODESAME recode);
+/**
+ * @brief Convert PROFILE enum to human readable text.
+ * @param[in] profile - PROFILE enum value to convert.
+ * @return PROFILE enum as text or "INVALID" if not known.
+ */
+std::string 	get_profile_text(PROFILE profile);
+/**
+ * @brief Convert PRORESLEVEL enum to human readable text.
+ * @param[in] level - PRORESLEVEL enum value to convert.
+ * @return PRORESLEVEL enum as text or "INVALID" if not known.
+ */
+std::string 	get_level_text(PRORESLEVEL level);
+/**
+ * @brief Get the selected hardware acceleration as text.
+ * @param[in] hwaccel_API - Hardware acceleration buffering API.
+ * @return Hardware acceleration API as string.
+ */
+std::string  	get_hwaccel_API_text(HWACCELAPI hwaccel_API);
+
+/**
+ * @brief Check if codec_id and optional profile is in block list.
+ * @param codec_id - Codec ID of rto check
+ * @param profile - Profile to check. Set to FF_PROFILE_UNKOWN to ignore.
+ * @return Returns true if codec is in block list, false if not.
+ */
+bool            check_hwaccel_dec_blocked(AVCodecID codec_id, int profile);
+
 /**
  * @brief Wrapper to the Fuse filler function.
- * @param buf[inout] - The buffer passed to the readdir() operation. May be nullptr.
- * @param filler[in] - Function pointer to the Fuse update function.  May be nullptr.
- * @param name[in] - The file name of the directory entry. Do not include the path!
- * @param stat[in] - File attributes, can be nullptr.
- * @param off[in] - Offset of the next entry or zero.
+ * @param[inout] buf - The buffer passed to the readdir() operation. May be nullptr.
+ * @param[in] filler - Function pointer to the Fuse update function.  May be nullptr.
+ * @param[in] name - The file name of the directory entry. Do not include the path!
+ * @param[in] stbuf - File attributes, can be nullptr.
+ * @param[in] off - Offset of the next entry or zero.
  * @return 1 if buffer is full, zero otherwise or if buf or filler is nullptr.
  */
 int             add_fuse_entry(void *buf, fuse_fill_dir_t filler, const char * name, const struct stat *stbuf, off_t off);
 
 /**
  * @brief Create the ./.. entries for a virtual directory
- * @param buf[inout] - The buffer passed to the readdir() operation. May be nullptr.
- * @param filler[in] - Function pointer to the Fuse update function.  May be nullptr.
- * @param stat[in] - File attributes, can be nullptr.
- * @param off[in] - Offset of the next entry or zero.
+ * @param[inout] buf - The buffer passed to the readdir() operation. May be nullptr.
+ * @param[in] filler - Function pointer to the Fuse update function.  May be nullptr.
+ * @param[in] stbuf - File attributes, can be nullptr.
+ * @param[in] off - Offset of the next entry or zero.
  * @return 1 if buffer is full, zero otherwise or if buf or filler is nullptr.
  */
 int             add_dotdot(void *buf, fuse_fill_dir_t filler, const struct stat *stbuf, off_t off);
