@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2017 Original author K. Henriksson
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,14 @@
  */
 
 /**
- * @file
+ * @file logging.cc
  * @brief Log facilities implementation
  *
  * @ingroup ffmpegfs
  *
  * @author K. Henriksson, Norbert Schlia (nschlia@oblivion-software.de)
  * @copyright Copyright (C) 2017 Original author K. Henriksson @n
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "logging.h"
@@ -62,6 +62,8 @@ namespace
 Logging* logging;
 }
 
+std::recursive_mutex Logging::m_mutex;
+
 Logging::Logging(const std::string &logfile, level max_level, bool to_stderr, bool to_syslog) :
     m_max_level(max_level),
     m_to_stderr(to_stderr),
@@ -85,14 +87,17 @@ Logging::Logger::~Logger()
         return;
     }
 
+    std::lock_guard<std::recursive_mutex> lck (m_mutex);
+
     // Construct string containing time
     time_t now = time(nullptr);
+    struct tm buffer;
     std::string time_string(30, '\0');
     std::string loglevel;
     std::string filename;
     std::string msg;
 
-    time_string.resize(strftime(&time_string[0], time_string.size(), "%F %T", localtime(&now)));   // Mind the blank at the end
+    time_string.resize(strftime(&time_string[0], time_string.size(), "%F %T", localtime_r(&now, &buffer)));   // Mind the blank at the end
 
     loglevel = m_level_name_map.at(m_loglevel) + ":";
 
@@ -121,6 +126,11 @@ Logging::Logger::~Logger()
             filename = COLOUR_LIGHT_PURPLE + filename + COLOUR_RESET;
         }
 
+        if (m_loglevel <= LOGERROR)
+        {
+            msg = COLOUR_LIGHT_RED + msg + COLOUR_RESET;
+        }
+
         std::clog << COLOUR_DARK_GRAY << time_string << " " << loglevel << COLOUR_RESET << " " << filename << msg << std::endl;
     }
 }
@@ -132,29 +142,29 @@ bool Logging::GetFail() const
 
 const std::map<Logging::level, int> Logging::Logger::m_syslog_level_map =
 {
-    { ERROR,     LOG_ERR },
-    { WARNING,   LOG_WARNING },
-    { INFO,      LOG_INFO },
-    { DEBUG,     LOG_DEBUG },
-    { TRACE,     LOG_DEBUG },
+    { LOGERROR,     LOG_ERR },
+    { LOGWARN,      LOG_WARNING },
+    { LOGINFO,      LOG_INFO },
+    { LOGDEBUG,     LOG_DEBUG },
+    { LOGTRACE,     LOG_DEBUG },
 };
 
 const std::map<Logging::level, std::string> Logging::Logger::m_level_name_map =
 {
-    { ERROR,     "ERROR  " },
-    { WARNING,   "WARNING" },
-    { INFO,      "INFO   " },
-    { DEBUG,     "DEBUG  " },
-    { TRACE,     "TRACE  " },
+    { LOGERROR,     "ERROR  " },
+    { LOGWARN,      "WARNING" },
+    { LOGINFO,      "INFO   " },
+    { LOGDEBUG,     "DEBUG  " },
+    { LOGTRACE,     "TRACE  " },
 };
 
 const std::map<Logging::level, std::string> Logging::Logger::m_level_colour_map =
 {
-    { ERROR,     COLOUR_RED },
-    { WARNING,   COLOUR_YELLOW },
-    { INFO,      COLOUR_WHITE },
-    { DEBUG,     COLOUR_GREEN },
-    { TRACE,     COLOUR_BLUE },
+    { LOGERROR,     COLOUR_RED },
+    { LOGWARN,      COLOUR_YELLOW },
+    { LOGINFO,      COLOUR_WHITE },
+    { LOGDEBUG,     COLOUR_GREEN },
+    { LOGTRACE,     COLOUR_BLUE },
 };
 
 Logging::Logger Log(Logging::level loglevel, const std::string & filename)
