@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2017 Original author K. Henriksson @n
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,14 +21,14 @@
  */
 
 /**
- * @file
+ * @file logging.h
  * @brief Provide various log facilities to stderr, disk or syslog
  *
  * @ingroup ffmpegfs
  *
  * @author K. Henriksson, Norbert Schlia (nschlia@oblivion-software.de)
  * @copyright Copyright (C) 2017 Original author K. Henriksson @n
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #ifndef LOGGING_H
@@ -40,6 +40,8 @@
 #include <fstream>
 #include <sstream>
 #include <regex>
+#include <mutex>
+
 /**
  * @brief #Logging facility
  *
@@ -77,16 +79,16 @@ public:
      */
     enum class level
     {
-        ERROR = 1,      /**< @brief Error level */
-        WARNING = 2,    /**< @brief Warning level */
-        INFO = 3,       /**< @brief Info level */
-        DEBUG = 4,      /**< @brief Debug level */
-        TRACE = 5       /**< @brief Error level */
+        LOGERROR = 1,      /**< @brief Error level */
+        LOGWARN = 2,    /**< @brief Warning level */
+        LOGINFO = 3,       /**< @brief Info level */
+        LOGDEBUG = 4,      /**< @brief Debug level */
+        LOGTRACE = 5       /**< @brief Error level */
     };
 
     /**
      * Construct Logging object
-	 *
+     *
      * @param[in] logfile - The name of a file to write logging output to. If empty, no output will be written.
      * @param[in] max_level - The maximum level of log output to write.
      * @param[in] to_stderr - Whether to write log output to stderr.
@@ -123,7 +125,8 @@ private:
          * @brief Construct Logger object
          */
         explicit Logger() :
-            m_loglevel(level::DEBUG) {}
+            m_loglevel(level::LOGDEBUG),
+            m_logging(nullptr) {}
         /**
          * @brief Destroy Logger object
          */
@@ -161,7 +164,7 @@ public:
     template <typename... Args>
     static void trace(const char *filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::TRACE, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGTRACE, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
     }
     /**
      * @brief Write trace level log entry
@@ -172,7 +175,7 @@ public:
     template <typename... Args>
     static void trace(const std::string &filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::TRACE, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGTRACE, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
     }
 
     /**
@@ -184,7 +187,7 @@ public:
     template <typename... Args>
     static void debug(const char * filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::DEBUG, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGDEBUG, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
     }
     /**
      * @brief Write debug level log entry
@@ -195,7 +198,7 @@ public:
     template <typename... Args>
     static void debug(const std::string & filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::DEBUG, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGDEBUG, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
     }
 
     /**
@@ -207,7 +210,7 @@ public:
     template <typename... Args>
     static void info(const char *filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::INFO, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGINFO, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
     }
     /**
      * @brief Write info level log entry
@@ -218,7 +221,7 @@ public:
     template <typename... Args>
     static void info(const std::string &filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::INFO, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGINFO, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
     }
 
     /**
@@ -230,7 +233,7 @@ public:
     template <typename... Args>
     static void warning(const char *filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::WARNING, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGWARN, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
     }
     /**
      * @brief Write warning level log entry
@@ -241,7 +244,7 @@ public:
     template <typename... Args>
     static void warning(const std::string &filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::WARNING, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGWARN, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
     }
 
     /**
@@ -253,7 +256,7 @@ public:
     template <typename... Args>
     static void error(const char *filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::ERROR, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGERROR, filename != nullptr ? filename : "", format_helper(format_string, 1, std::forward<Args>(args)...));
     }
     /**
      * @brief Write error level log entry
@@ -264,7 +267,7 @@ public:
     template <typename... Args>
     static void error(const std::string &filename, const std::string &format_string, Args &&...args)
     {
-        log_with_level(Logging::level::ERROR, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
+        log_with_level(Logging::level::LOGERROR, filename, format_helper(format_string, 1, std::forward<Args>(args)...));
     }
 
     /**
@@ -364,18 +367,19 @@ protected:
      * @param[in] filename - If logging to file, use this file name.
      */
     friend Logger Log(level loglevel, const std::string & filename);
-    friend Logger;                                                      /**< @brief Make logger class our friend */
+    friend Logger;                                                  /**< @brief Make logger class our friend */
 
-    std::ofstream   m_logfile;                      /**< @brief Log file object for writing to disk */
-    const level     m_max_level;                    /**< @brief The maximum level of log output to write. */
-    const bool      m_to_stderr;                    /**< @brief Whether to write log output to stderr. */
-    const bool      m_to_syslog;                    /**< @brief Whether to write log output to syslog. */
+    static std::recursive_mutex     m_mutex;                        /**< @brief Access mutex */
+    std::ofstream                   m_logfile;                      /**< @brief Log file object for writing to disk */
+    const level                     m_max_level;                    /**< @brief The maximum level of log output to write. */
+    const bool                      m_to_stderr;                    /**< @brief Whether to write log output to stderr. */
+    const bool                      m_to_syslog;                    /**< @brief Whether to write log output to syslog. */
 };
 
-constexpr Logging::level ERROR    = Logging::level::ERROR;    /**< @brief Shorthand for log level ERROR */
-constexpr Logging::level WARNING  = Logging::level::WARNING;  /**< @brief Shorthand for log level WARNING */
-constexpr Logging::level INFO     = Logging::level::INFO;     /**< @brief Shorthand for log level INFO */
-constexpr Logging::level DEBUG    = Logging::level::DEBUG;    /**< @brief Shorthand for log level DEBUG */
-constexpr Logging::level TRACE    = Logging::level::TRACE;    /**< @brief Shorthand for log level TRACE */
+constexpr Logging::level LOGERROR   = Logging::level::LOGERROR;     /**< @brief Shorthand for log level ERROR */
+constexpr Logging::level LOGWARN    = Logging::level::LOGWARN;      /**< @brief Shorthand for log level WARNING */
+constexpr Logging::level LOGINFO    = Logging::level::LOGINFO;      /**< @brief Shorthand for log level INFO */
+constexpr Logging::level LOGDEBUG   = Logging::level::LOGDEBUG;     /**< @brief Shorthand for log level DEBUG */
+constexpr Logging::level LOGTRACE   = Logging::level::LOGTRACE;     /**< @brief Shorthand for log level TRACE */
 
 #endif
