@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2019-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,13 +20,13 @@
  */
 
 /**
- * @file
+ * @file thread_pool.cc
  * @brief Thread pool class implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2019-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2019-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "thread_pool.h"
@@ -37,7 +37,7 @@ thread_pool::thread_pool(unsigned int num_threads)
     : m_queue_shutdown(false)
     , m_num_threads(num_threads)
     , m_cur_threads(0)
-
+    , m_threads_running(0)
 {
 }
 
