@@ -1,7 +1,7 @@
 ﻿/*
  * Copyright (C) 2006-2008 David Collett
  * Copyright (C) 2008-2013 K. Henriksson
- * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 /**
- * @file
+ * @file transcode.cc
  * @brief File transcoder interface implementation
  *
  * @ingroup ffmpegfs
@@ -30,7 +30,7 @@
  * @author Norbert Schlia (nschlia@oblivion-software.de)
  * @copyright Copyright (C) 2006-2008 David Collett @n
  * Copyright (C) 2008-2013 K. Henriksson @n
- * Copyright (C) 2017-2021 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 FFmpeg support by Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "transcode.h"
@@ -82,7 +82,7 @@ static bool transcode_until(Cache_Entry* cache_entry, size_t offset, size_t len,
     size_t end = offset + len; // Cast OK: offset will never be < 0.
     bool success = true;
 
-    if (cache_entry->m_cache_info.m_finished == RESULTCODE_FINISHED || cache_entry->m_buffer->is_segment_finished(segment_no) || cache_entry->m_buffer->tell(segment_no) >= end)
+    if (cache_entry->is_finished() || cache_entry->m_buffer->is_segment_finished(segment_no) || cache_entry->m_buffer->tell(segment_no) >= end)
     {
         return true;
     }
@@ -93,7 +93,7 @@ static bool transcode_until(Cache_Entry* cache_entry, size_t offset, size_t len,
         if (cache_entry->m_is_decoding)
         {
             bool reported = false;
-            while (!(cache_entry->m_cache_info.m_finished == RESULTCODE_FINISHED || cache_entry->m_buffer->is_segment_finished(segment_no) || cache_entry->m_buffer->tell(segment_no) >= end) && !cache_entry->m_cache_info.m_error)
+            while (!(cache_entry->is_finished() || cache_entry->m_buffer->is_segment_finished(segment_no) || cache_entry->m_buffer->tell(segment_no) >= end) && !cache_entry->m_cache_info.m_error)
             {
                 if (fuse_interrupted())
                 {
@@ -109,7 +109,7 @@ static bool transcode_until(Cache_Entry* cache_entry, size_t offset, size_t len,
 
                 if (!reported)
                 {
-                    Logging::trace(cache_entry->destname(), "Cache miss at offset %<%11zu>1 (length %<%6u>2), remaining %3.", offset, len, format_size_ex(cache_entry->m_buffer->size(segment_no) - end).c_str());
+                    Logging::trace(cache_entry->destname(), "Segment no. %1: Cache miss at offset %2 (length %3), remaining %4.", segment_no, offset, len, format_size_ex(end - cache_entry->m_buffer->size(segment_no)).c_str());
                     reported = true;
                 }
                 sleep(0);
@@ -117,7 +117,7 @@ static bool transcode_until(Cache_Entry* cache_entry, size_t offset, size_t len,
 
             if (reported)
             {
-                Logging::trace(cache_entry->destname(), "Cache hit  at offset %<%11zu>1 (length %<%6u>2), remaining %3.", offset, len, format_size_ex(cache_entry->m_buffer->size(segment_no) - end).c_str());
+                Logging::trace(cache_entry->destname(), "Segment no. %1: Cache hit at offset %2 (length %3), remaining %4.", segment_no, offset, len, format_size_ex(end - cache_entry->m_buffer->size(segment_no)).c_str());
             }
             success = !cache_entry->m_cache_info.m_error;
         }
@@ -149,7 +149,7 @@ static int transcode_finish(Cache_Entry* cache_entry, FFmpeg_Transcoder &transco
     cache_entry->m_cache_info.m_encoded_filesize    = cache_entry->m_buffer->buffer_watermark();
     cache_entry->m_cache_info.m_video_frame_count   = transcoder.video_frame_count();
     cache_entry->m_cache_info.m_segment_count       = transcoder.segment_count();
-    cache_entry->m_cache_info.m_finished            = RESULTCODE_FINISHED;
+    cache_entry->m_cache_info.m_result              = !transcoder.have_seeked() ? RESULTCODE_FINISHED_SUCCESS : RESULTCODE_FINISHED_INCOMPLETE;
     cache_entry->m_is_decoding                      = false;
     cache_entry->m_cache_info.m_errno               = 0;
     cache_entry->m_cache_info.m_averror             = 0;
@@ -270,37 +270,42 @@ bool transcoder_cached_filesize(LPVIRTUALFILE virtualfile, struct stat *stbuf)
     }
 }
 
-bool transcoder_set_filesize(LPVIRTUALFILE virtualfile, int64_t duration, BITRATE audio_bit_rate, int channels, int sample_rate, BITRATE video_bit_rate, int width, int height, int interleaved, const AVRational &framerate)
+bool transcoder_set_filesize(LPVIRTUALFILE virtualfile, int64_t duration, BITRATE audio_bit_rate, int channels, int sample_rate, AVSampleFormat sample_format, BITRATE video_bit_rate, int width, int height, int interleaved, const AVRational &framerate)
 {
     Cache_Entry* cache_entry = cache->open(virtualfile);
     if (cache_entry == nullptr)
     {
-        Logging::error(cache_entry->filename(), "Out of memory getting file size.");
+        Logging::error(nullptr, "Out of memory getting file size.");
         return false;
     }
 
     FFmpegfs_Format *current_format = params.current_format(virtualfile);
     if (current_format == nullptr)
     {
-        Logging::error(cache_entry->filename(), "Internal error getting file size.");
+        Logging::error(cache_entry->destname(), "Internal error getting file size.");
         return false;
     }
 
     size_t filesize = 0;
 
-    if (!FFmpeg_Transcoder::audio_size(&filesize, current_format->audio_codec_id(), audio_bit_rate, duration, channels, sample_rate))
+    if (!FFmpeg_Transcoder::audio_size(&filesize, current_format->audio_codec(), audio_bit_rate, duration, channels, sample_rate, sample_format))
     {
-        Logging::warning(cache_entry->filename(), "Unsupported audio codec '%1' for format %2.", get_codec_name(current_format->audio_codec_id(), 0), current_format->desttype().c_str());
+        Logging::warning(cache_entry->destname(), "Unsupported audio codec '%1' for format %2.", get_codec_name(current_format->audio_codec(), 0), current_format->desttype().c_str());
     }
 
-    if (!FFmpeg_Transcoder::video_size(&filesize, current_format->video_codec_id(), video_bit_rate, duration, width, height, interleaved, framerate))
+    if (!FFmpeg_Transcoder::video_size(&filesize, current_format->video_codec(), video_bit_rate, duration, width, height, interleaved, framerate))
     {
-        Logging::warning(cache_entry->filename(), "Unsupported video codec '%1' for format %2.", get_codec_name(current_format->video_codec_id(), 0), current_format->desttype().c_str());
+        Logging::warning(cache_entry->destname(), "Unsupported video codec '%1' for format %2.", get_codec_name(current_format->video_codec(), 0), current_format->desttype().c_str());
     }
 
-    cache_entry->m_cache_info.m_predicted_filesize = filesize;
+    if (!FFmpeg_Transcoder::total_overhead(&filesize, current_format->filetype()))
+    {
+        Logging::warning(cache_entry->destname(), "Unsupported file type '%1' for format %2.", get_filetype_text(current_format->filetype()).c_str(), current_format->desttype().c_str());
+    }
 
-    Logging::trace(cache_entry->filename(), "Predicted transcoded size of %1.", format_size_ex(cache_entry->m_cache_info.m_predicted_filesize).c_str());
+    cache_entry->m_cache_info.m_predicted_filesize = virtualfile->m_predicted_size = filesize;
+
+    Logging::trace(cache_entry->destname(), "Predicted transcoded size of %1.", format_size_ex(cache_entry->m_cache_info.m_predicted_filesize).c_str());
 
     return true;
 }
@@ -372,19 +377,14 @@ Cache_Entry* transcoder_new(LPVIRTUALFILE virtualfile, bool begin_transcode)
             virtualfile->m_video_frame_count    = cache_entry->m_cache_info.m_video_frame_count;
         }
 
-        if (!cache_entry->m_is_decoding && cache_entry->m_cache_info.m_finished != RESULTCODE_FINISHED)
+        if (!cache_entry->m_is_decoding && !cache_entry->is_finished_success())
         {
             if (begin_transcode)
             {
-                int ret;
+                Logging::debug(cache_entry->filename(), "Starting transcoder thread.");
 
-                Logging::debug(cache_entry->filename(), "Starting decoder thread.");
-
-                if (cache_entry->m_cache_info.m_error)
-                {
-                    // If error occurred last time, clear cache
-                    cache_entry->clear();
-                }
+                // Clear cache to remove any older remains
+                cache_entry->clear();
 
                 // Must decode the file, otherwise simply use cache
                 cache_entry->m_is_decoding  = true;
@@ -409,7 +409,10 @@ Cache_Entry* transcoder_new(LPVIRTUALFILE virtualfile, bool begin_transcode)
 
                 if (cache_entry->m_cache_info.m_error)
                 {
-                    Logging::trace(cache_entry->filename(), "Decoder error!");
+                    int ret;
+
+                    Logging::trace(cache_entry->filename(), "Transcoder error!");
+
                     ret = cache_entry->m_cache_info.m_errno;
                     if (!ret)
                     {
@@ -418,7 +421,7 @@ Cache_Entry* transcoder_new(LPVIRTUALFILE virtualfile, bool begin_transcode)
                     throw ret;
                 }
 
-                Logging::debug(cache_entry->filename(), "Decoder thread is running.");
+                Logging::debug(cache_entry->filename(), "Transcoder thread is running.");
             }
             else if (!cache_entry->m_cache_info.m_predicted_filesize)
             {
@@ -435,19 +438,18 @@ Cache_Entry* transcoder_new(LPVIRTUALFILE virtualfile, bool begin_transcode)
 
         cache_entry->unlock();
     }
-    catch (int _errno)
+    catch (int orgerrno)
     {
         cache_entry->m_is_decoding = false;
         cache_entry->unlock();
         cache->close(&cache_entry, CACHE_CLOSE_DELETE);
         cache_entry = nullptr;  // Make sure to return NULL here even if the cache could not be deleted now (still in use)
-        errno = _errno;         // Restore last errno
+        errno = orgerrno;         // Restore last errno
     }
 
     return cache_entry;
 }
 
-#define MIN_SEGMENT 3   /**< @brief Seek new segment if more distant than this range @todo Seek new segment if more distance should better count in seconds, not segments... */
 bool transcoder_read(Cache_Entry* cache_entry, char* buff, size_t offset, size_t len, int * bytes_read, uint32_t segment_no)
 {
     bool success = true;
@@ -471,7 +473,7 @@ bool transcoder_read(Cache_Entry* cache_entry, char* buff, size_t offset, size_t
     {
         // Try to read requested segment, stack a seek to if if this fails.
         // No seek if not HLS (segment_no) and not required if < MIN_SEGMENT
-        if (segment_no > MIN_SEGMENT && !cache_entry->m_buffer->segment_exists(segment_no))
+        if (segment_no && !cache_entry->m_buffer->segment_exists(segment_no))
         {
             cache_entry->m_seek_to_no = segment_no;
         }
@@ -482,7 +484,7 @@ bool transcoder_read(Cache_Entry* cache_entry, char* buff, size_t offset, size_t
             throw false;
         }
 
-        if (cache_entry->m_cache_info.m_finished != RESULTCODE_FINISHED)
+        if (!cache_entry->is_finished_success())
         {
             switch (params.current_format(cache_entry->virtualfile())->filetype())
             {
@@ -536,10 +538,7 @@ bool transcoder_read(Cache_Entry* cache_entry, char* buff, size_t offset, size_t
             }
         }
 
-        // Set last access time
-        cache_entry->m_cache_info.m_access_time = time(nullptr);
-
-        bool success = transcode_until(cache_entry, offset, len, segment_no);
+        success = transcode_until(cache_entry, offset, len, segment_no);
 
         if (!success)
         {
@@ -602,9 +601,6 @@ bool transcoder_read_frame(Cache_Entry* cache_entry, char* buff, size_t offset,
             throw false;
         }
 
-        // Set last access time
-        cache_entry->m_cache_info.m_access_time = time(nullptr);
-
         std::vector<uint8_t> data;
 
         // Wait until decoder thread has the requested frame available
@@ -615,10 +611,15 @@ bool transcoder_read_frame(Cache_Entry* cache_entry, char* buff, size_t offset,
             // Try to read requested frame, stack a seek to if if this fails.
             if (!cache_entry->m_buffer->read_frame(&data, frame_no))
             {
+                Logging::error(nullptr, "SEEK image frame no. %1", frame_no);
                 cache_entry->m_seek_to_no = frame_no;
             }
+            else if (!offset)
+                Logging::error(nullptr, "NOSEEK image frame no. %1", frame_no);
 
-            int retries = 120; // wait 120 x 250 ms = 30 sec.
+#define GRANULARITY     250 // ms
+#define FRAME_TIMEOUT   10  // seconds
+            int retries = FRAME_TIMEOUT * 1000 / GRANULARITY;
             while (!cache_entry->m_buffer->read_frame(&data, frame_no) && !thread_exit)
             {
                 if (errno != EAGAIN)
@@ -652,13 +653,13 @@ bool transcoder_read_frame(Cache_Entry* cache_entry, char* buff, size_t offset,
                     reported = true;
                 }
 
-                const struct timespec ts = { 0, 250 MS };
+                const struct timespec ts = { 0, GRANULARITY MS };
                 nanosleep(&ts, nullptr);
             }
 
             if (reported)
             {
-                Logging::trace(cache_entry->destname(), "Cache hit  at offset %<%11zu>1 (length %<%6u>2).", offset, len);
+                Logging::trace(cache_entry->destname(), "Frame no. %1: Cache hit  at offset %<%11zu>2 (length %<%6u>3).", frame_no, offset, len);
             }
             success = !cache_entry->m_cache_info.m_error;
         }
@@ -765,8 +766,16 @@ static void transcoder_thread(void *arg)
 
     std::unique_lock<std::recursive_mutex> lock(cache_entry->m_active_mutex);
 
+    // Clear cache to remove any older remains
+    cache_entry->clear();
+
+    // Must decode the file, otherwise simply use cache
+    cache_entry->m_is_decoding  = true;
+
     try
     {
+        bool unlocked = false;
+
         Logging::info(cache_entry->filename(), "Transcoding to %1.", params.current_format(cache_entry->virtualfile())->desttype().c_str());
 
         if (!cache_entry->open())
@@ -815,7 +824,7 @@ static void transcoder_thread(void *arg)
 
         thread_data->m_initialised = true;
 
-        bool unlocked = false;
+        unlocked = false;
         if (!params.m_prebuffer_size || transcoder.is_frameset())
         {
             // Unlock frame set from beginning
@@ -828,7 +837,7 @@ static void transcoder_thread(void *arg)
             Logging::debug(cache_entry->destname(), "Pre-buffering up to %1 bytes.", params.m_prebuffer_size);
         }
 
-        while ((cache_entry->m_cache_info.m_finished != RESULTCODE_FINISHED) && !(timeout = cache_entry->decode_timeout()) && !thread_exit)
+        while (!cache_entry->is_finished() && !(timeout = cache_entry->decode_timeout()) && !thread_exit)
         {
             int status = 0;
 
@@ -870,16 +879,14 @@ static void transcoder_thread(void *arg)
             averror = transcoder.process_single_fr(status);
             if (status < 0)
             {
-                syserror = EIO;
-                success = false;
-                break;
+                errno = EIO;
+                throw (static_cast<int>(errno));
             }
 
             if (status == 1 && ((averror = transcode_finish(cache_entry, transcoder)) < 0))
             {
-                syserror = EIO;
-                success = false;
-                break;
+                errno = EIO;
+                throw (static_cast<int>(errno));
             }
 
             if (!unlocked && cache_entry->m_buffer->buffer_watermark() > params.m_prebuffer_size)
@@ -903,7 +910,8 @@ static void transcoder_thread(void *arg)
 
                 while (cache_entry->suspend_timeout() && !(timeout = cache_entry->decode_timeout()) && !thread_exit)
                 {
-                    sleep(1);
+                    const struct timespec ts = { 0, GRANULARITY MS };
+                    nanosleep(&ts, nullptr);
                 }
 
                 if (timeout)
@@ -927,10 +935,17 @@ static void transcoder_thread(void *arg)
         success = false;
         syserror = _syserror;
 
+        if (!syserror && averror > -512)
+        {
+            // If no system error reported explicitly, and averror is a POSIX error
+            // (we simply assume that if averror < 512, I think averrors are all higher values)
+            syserror = AVUNERROR(averror);
+        }
+
         cache_entry->m_is_decoding              = false;
-        cache_entry->m_cache_info.m_error       = !success;
-        cache_entry->m_cache_info.m_errno       = success ? 0 : (syserror ? syserror : EIO);    // Preserve errno
-        cache_entry->m_cache_info.m_averror     = success ? 0 : averror;                        // Preserve averror
+        cache_entry->m_cache_info.m_error       = true;
+        cache_entry->m_cache_info.m_errno       = syserror ? syserror : EIO;        // Preserve errno
+        cache_entry->m_cache_info.m_averror     = averror;                          // Preserve averror
 
         thread_data->m_lock_guard = true;
         thread_data->m_cond.notify_all();           // unlock main thread
@@ -944,7 +959,7 @@ static void transcoder_thread(void *arg)
 
         if (!transcoder.have_seeked())
         {
-            cache_entry->m_cache_info.m_finished    = RESULTCODE_ERROR;
+            //cache_entry->m_cache_info.m_finished    = RESULTCODE_FINISHED_ERROR;
             cache_entry->m_cache_info.m_error       = true;
             cache_entry->m_cache_info.m_errno       = EIO;      // Report I/O error
             cache_entry->m_cache_info.m_averror     = averror;  // Preserve averror
@@ -961,7 +976,7 @@ static void transcoder_thread(void *arg)
         else
         {
             // Must restart from scratch, but this is not an error.
-            cache_entry->m_cache_info.m_finished    = RESULTCODE_INCOMPLETE;
+            //cache_entry->m_cache_info.m_finished    = RESULTCODE_FINISHED_INCOMPLETE;
             cache_entry->m_cache_info.m_error       = false;
             cache_entry->m_cache_info.m_errno       = 0;
             cache_entry->m_cache_info.m_averror     = 0;
@@ -1002,7 +1017,7 @@ static void transcoder_thread(void *arg)
 void ffmpeg_log(void *ptr, int level, const char *fmt, va_list vl)
 {
     va_list vl2;
-    Logging::level ffmpegfs_level = ERROR;
+    Logging::level ffmpegfs_level;
     static int print_prefix = 1;
 
 #if (LIBAVUTIL_VERSION_INT >= AV_VERSION_INT(55, 23, 0))
@@ -1014,6 +1029,7 @@ void ffmpeg_log(void *ptr, int level, const char *fmt, va_list vl)
     line_size = av_log_format_line2(ptr, level, fmt, vl2, nullptr, 0, &print_prefix);
     if (line_size < 0)
     {
+        va_end(vl2);
         return;
     }
     line = static_cast<char *>(av_malloc(static_cast<size_t>(line_size)));
@@ -1038,29 +1054,29 @@ void ffmpeg_log(void *ptr, int level, const char *fmt, va_list vl)
     // AV_LOG_ERROR    16
     if (level <= AV_LOG_ERROR)
     {
-        ffmpegfs_level = ERROR;
+        ffmpegfs_level = LOGERROR;
     }
     // AV_LOG_WARNING  24
     else if (level <= AV_LOG_WARNING)
     {
-        ffmpegfs_level = WARNING;
+        ffmpegfs_level = LOGWARN;
     }
 #ifdef AV_LOG_TRACE
     // AV_LOG_INFO     32
     else if (level <= AV_LOG_INFO)
     {
-        ffmpegfs_level = INFO;
+        ffmpegfs_level = LOGINFO;
     }
     // AV_LOG_VERBOSE  40
     // AV_LOG_DEBUG    48
     else if (level <= AV_LOG_DEBUG)
     {
-        ffmpegfs_level = DEBUG;
+        ffmpegfs_level = LOGDEBUG;
     }
     // AV_LOG_TRACE    56
     else   // if (level <= AV_LOG_TRACE)
     {
-        ffmpegfs_level = TRACE;
+        ffmpegfs_level = LOGTRACE;
     }
 #else
     // AV_LOG_INFO     32
@@ -1079,16 +1095,16 @@ void ffmpeg_log(void *ptr, int level, const char *fmt, va_list vl)
 
 bool init_logging(const std::string &logfile, const std::string & max_level, bool to_stderr, bool to_syslog)
 {
-    static const std::map<std::string, Logging::level, comp> level_map =
+    static const std::map<const std::string, const Logging::level, comp> level_map =
     {
-        { "ERROR",      ERROR },
-        { "WARNING",    WARNING },
-        { "INFO",       INFO },
-        { "DEBUG",      DEBUG },
-        { "TRACE",      TRACE },
+        { "ERROR",      LOGERROR },
+        { "WARNING",    LOGWARN },
+        { "INFO",       LOGINFO },
+        { "DEBUG",      LOGDEBUG },
+        { "TRACE",      LOGTRACE },
     };
 
-    std::map<std::string, Logging::level, comp>::const_iterator it = level_map.find(max_level);
+    std::map<const std::string, const Logging::level, comp>::const_iterator it = level_map.find(max_level);
 
     if (it == level_map.cend())
     {
