@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,14 +17,14 @@
  */
 
 /**
- * @file
+ * @file vcdchapter.cc
  * @brief S/VCD VcdChapter class implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2013-2021 Norbert Schlia (nschlia@oblivion-software.de) @n
- * From BullysPLayer Copyright (C) 1984-2021 by Oblivion Software/Norbert Schlia
+ * @copyright Copyright (C) 2013-2022 Norbert Schlia (nschlia@oblivion-software.de) @n
+ * From BullysPLayer Copyright (C) 1984-2022 by Oblivion Software/Norbert Schlia
  */
 
 #include "vcdchapter.h"
@@ -41,7 +41,8 @@ VcdChapter::VcdChapter(bool is_svcd) :
     m_sec(0),
     m_frame(0),
     m_duration(0),
-    m_start_pos(0)
+    m_start_pos(0),
+    m_end_pos(0)
 {
 
 }
@@ -53,7 +54,8 @@ VcdChapter::VcdChapter(const VCDCHAPTER & VcdChapter, bool is_svcd) :
     m_sec(BCD2DEC(VcdChapter.m_msf.m_sec)),
     m_frame(BCD2DEC(VcdChapter.m_msf.m_frame)),
     m_duration(0),
-    m_start_pos(0)
+    m_start_pos(0),
+    m_end_pos(0)
 {
 
 }
@@ -145,11 +147,11 @@ std::string VcdChapter::get_filename() const
 
     if (m_is_svcd)
     {
-        snprintf(buffer, sizeof(buffer) - 1, "MPEG2/AVSEQ%02u.MPG", m_track_no - 1);
+        snprintf(buffer, sizeof(buffer) - 1, "MPEG2/AVSEQ%02i.MPG", m_track_no - 1);
     }
     else
     {
-        snprintf(buffer, sizeof(buffer) - 1, "MPEGAV/AVSEQ%02u.DAT", m_track_no - 1);
+        snprintf(buffer, sizeof(buffer) - 1, "MPEGAV/AVSEQ%02i.DAT", m_track_no - 1);
     }
     return buffer;
 }
@@ -203,6 +205,7 @@ VcdChapter & VcdChapter::operator= (VcdChapter const & other)
         m_sec       = other.m_sec;
         m_frame     = other.m_frame;
         m_start_pos = other.m_start_pos;
+        m_end_pos   = other.m_end_pos;
         m_duration  = other.m_duration;
     }
 
@@ -220,6 +223,7 @@ int VcdChapter::operator==(const VcdChapter & other) const
 int VcdChapter::operator<(const VcdChapter & other) const
 {
     int res;
+
     res = (m_track_no - other.m_track_no);
 
     if (res < 0)
@@ -263,7 +267,6 @@ int VcdChapter::operator<(const VcdChapter & other) const
         return 1;
     }
 
-    //if (res >= 0)
     return 0;
 }
 
