@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,14 +17,14 @@
  */
 
 /**
- * @file
+ * @file vcdentries.cc
  * @brief S/VCD VcdEntries class implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2013-2021 Norbert Schlia (nschlia@oblivion-software.de) @n
- * From BullysPLayer Copyright (C) 1984-2021 by Oblivion Software/Norbert Schlia
+ * @copyright Copyright (C) 2013-2022 Norbert Schlia (nschlia@oblivion-software.de) @n
+ * From BullysPLayer Copyright (C) 1984-2022 by Oblivion Software/Norbert Schlia
  */
 
 #include <string>
@@ -155,13 +155,13 @@ int VcdEntries::load_file(const std::string & path)
             m_duration += chapter_duration;
         }
     }
-    catch (int _errno)
+    catch (int orgerrno)
     {
         if (fpi != nullptr)
         {
             fclose(fpi);
         }
-        return _errno;
+        return orgerrno;
     }
 
     fclose(fpi);
@@ -173,8 +173,6 @@ int VcdEntries::scan_chapters()
 {
     FILE *      fpi = nullptr;
     std::string fullname;
-    int         last_track_no = -1;
-    int64_t     first_sync = -1;
     struct stat stbuf;
 
     if (!m_chapters.size())
@@ -184,6 +182,9 @@ int VcdEntries::scan_chapters()
 
     try
     {
+        int         last_track_no = -1;
+        int64_t     first_sync = -1;
+
         // Build list of chapters
         for (size_t chapter_no = 0; chapter_no < m_chapters.size(); chapter_no++)
         {
@@ -191,10 +192,10 @@ int VcdEntries::scan_chapters()
             {
                 last_track_no = m_chapters[chapter_no].get_track_no();
 
-                int _errno = VCDUTILS::locate_video(m_disk_path, last_track_no, fullname);
-                if (_errno != 0)
+                int orgerrno = VCDUTILS::locate_video(m_disk_path, last_track_no, fullname);
+                if (orgerrno != 0)
                 {
-                    throw static_cast<int>(_errno);
+                    throw static_cast<int>(orgerrno);
                 }
 
                 if (chapter_no)
@@ -246,10 +247,10 @@ int VcdEntries::scan_chapters()
                     throw static_cast<int>(ferror(fpi));
                 }
 
-                int _errno = buffer.read(fpi, last_track_no);
-                if (_errno)
+                int orgerrno = buffer.read(fpi, last_track_no);
+                if (orgerrno)
                 {
-                    throw static_cast<int>(_errno);
+                    throw static_cast<int>(orgerrno);
                 }
 
                 if (buffer < m_chapters[chapter_no])
@@ -285,10 +286,10 @@ int VcdEntries::scan_chapters()
                 throw static_cast<int>(ferror(fpi));
             }
 
-            int _errno = buffer.read(fpi, last_track_no);
-            if (_errno)
+            int orgerrno = buffer.read(fpi, last_track_no);
+            if (orgerrno)
             {
-                throw static_cast<int>(_errno);
+                throw static_cast<int>(orgerrno);
             }
 
             VcdChapter & chapter1 = m_chapters[m_chapters.size() - 1];
@@ -300,13 +301,13 @@ int VcdEntries::scan_chapters()
             m_duration += chapter_duration;
         }
     }
-    catch (int _errno)
+    catch (int orgerrno)
     {
         if (fpi != nullptr)
         {
             fclose(fpi);
         }
-        return _errno;
+        return orgerrno;
     }
 
     // End of last chapter
