@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,14 +17,14 @@
  */
 
 /**
- * @file
+ * @file vcdinfo.cc
  * @brief S/VCD VcdInfo class implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2013-2021 Norbert Schlia (nschlia@oblivion-software.de) @n
- * From BullysPLayer Copyright (C) 1984-2021 by Oblivion Software/Norbert Schlia
+ * @copyright Copyright (C) 2013-2022 Norbert Schlia (nschlia@oblivion-software.de) @n
+ * From BullysPLayer Copyright (C) 1984-2022 by Oblivion Software/Norbert Schlia
  */
 
 #include <string>
@@ -183,16 +183,16 @@ int VcdInfo::load_file(const std::string & path)
         success = false;
     }
 
-    int _errno = 0;
+    int orgerrno = 0;
 
     if (success)
     {
-        _errno = ferror(fpi);
+        orgerrno = ferror(fpi);
     }
 
     fclose(fpi);
 
-    return _errno;
+    return orgerrno;
 }
 
 const time_t &VcdInfo::get_file_date() const
