@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,14 +17,14 @@
  */
 
 /**
- * @file
+ * @file vcdutils.cc
  * @brief S/VCD utility functions implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2013-2021 Norbert Schlia (nschlia@oblivion-software.de) @n
- * From BullysPLayer Copyright (C) 1984-2021 by Oblivion Software/Norbert Schlia
+ * @copyright Copyright (C) 2013-2022 Norbert Schlia (nschlia@oblivion-software.de) @n
+ * From BullysPLayer Copyright (C) 1984-2022 by Oblivion Software/Norbert Schlia
  */
 
 #include "vcdutils.h"
@@ -90,7 +90,7 @@ int locate_video(const std::string & path, int track_no, std::string & fullname)
     char buffer[PATH_MAX + 1];
 
     // Try VCD
-    snprintf(buffer, sizeof(buffer) - 1, "MPEGAV/AVSEQ%02u.DAT", track_no - 1);
+    snprintf(buffer, sizeof(buffer) - 1, "MPEGAV/AVSEQ%02i.DAT", track_no - 1);
     fullname = path + buffer;
     if (!access(fullname.c_str(), F_OK))
     {
@@ -98,7 +98,7 @@ int locate_video(const std::string & path, int track_no, std::string & fullname)
     }
 
     // Try SVCD
-    snprintf(buffer, sizeof(buffer) - 1, "MPEG2/AVSEQ%02u.MPG", track_no - 1);
+    snprintf(buffer, sizeof(buffer) - 1, "MPEG2/AVSEQ%02i.MPG", track_no - 1);
     fullname = path + buffer;
     if (!access(fullname.c_str(), F_OK))
     {
