@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,13 +22,13 @@
 #ifdef USE_LIBVCD
 
 /**
- * @file
+ * @file vcdparser.cc
  * @brief Video/Super Video CD parser implementation
  *
  * @ingroup ffmpegfs
  *
  * @author Norbert Schlia (nschlia@oblivion-software.de)
- * @copyright Copyright (C) 2017-2021 Norbert Schlia (nschlia@oblivion-software.de)
+ * @copyright Copyright (C) 2017-2022 Norbert Schlia (nschlia@oblivion-software.de)
  */
 
 #include "ffmpegfs.h"
@@ -67,19 +67,19 @@ static bool create_vcd_virtualfile(const VcdEntries & vcd, const struct stat * s
     {
         size = static_cast<size_t>(chapter1->get_size());
         duration = chapter1->get_duration();
-        snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Chapter %03d [%s].%s", chapter1->get_track_no(), chapter_no + 1, replace_all(format_duration(duration), ":", "-").c_str(), params.m_format[0].fileext().c_str()); // can safely assume this a video
+        snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Chapter %03d [%s].%s", chapter1->get_track_no(), chapter_no + 1, replace_all(format_duration(duration), ":", "-").c_str(), ffmpeg_format[0].fileext().c_str()); // can safely assume this a video
     }
     else
     {
         size = static_cast<size_t>(vcd.get_size());
         duration = vcd.get_duration();
-        snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Title [%s].%s", chapter1->get_track_no(), replace_all(format_duration(duration), ":", "-").c_str(), params.m_format[0].fileext().c_str()); // can safely assume this a video
+        snprintf(title_buf, sizeof(title_buf) - 1, "%02d. Title [%s].%s", chapter1->get_track_no(), replace_all(format_duration(duration), ":", "-").c_str(), ffmpeg_format[0].fileext().c_str()); // can safely assume this a video
     }
 
     std::string filename(title_buf);
 
     LPVIRTUALFILE virtualfile = nullptr;
-    if (!params.m_format[0].is_multiformat())
+    if (!ffmpeg_format[0].is_multiformat())
     {
         virtualfile = insert_file(VIRTUALTYPE_VCD, vcd.get_disk_path() + filename, statbuf);
     }
@@ -160,40 +160,40 @@ static int parse_vcd(const std::string & path, const struct stat * statbuf, void
     }
 }
 
-int check_vcd(const std::string & _path, void *buf, fuse_fill_dir_t filler)
+int check_vcd(const std::string & path, void *buf, fuse_fill_dir_t filler)
 {
-    std::string path(_path);
+    std::string _path(path);
     struct stat stbuf;
     int res = 0;
 
-    append_sep(&path);
+    append_sep(&_path);
 
-    if (stat((path + "SVCD/INFO.SVD").c_str(), &stbuf) == 0)
+    if (stat((_path + "SVCD/INFO.SVD").c_str(), &stbuf) == 0)
     {
-        if (!check_path(path))
+        if (!check_path(_path))
         {
-            Logging::trace(path, "SVCD detected.");
-            res = parse_vcd(path, &stbuf, buf, filler);
+            Logging::trace(_path, "SVCD detected.");
+            res = parse_vcd(_path, &stbuf, buf, filler);
             Logging::trace(nullptr, "Found %1 titles.", res);
         }
         else
         {
-            res = load_path(path, &stbuf, buf, filler);
+            res = load_path(_path, &stbuf, buf, filler);
         }
 
         add_dotdot(buf, filler, &stbuf, 0);
     }
-    else if (stat((path + "VCD/INFO.VCD").c_str(), &stbuf) == 0)
+    else if (stat((_path + "VCD/INFO.VCD").c_str(), &stbuf) == 0)
     {
-        if (!check_path(path))
+        if (!check_path(_path))
         {
-            Logging::trace(path, "VCD detected.");
-            res = parse_vcd(path, &stbuf, buf, filler);
+            Logging::trace(_path, "VCD detected.");
+            res = parse_vcd(_path, &stbuf, buf, filler);
             Logging::trace(nullptr, "Found %1 titles.", res);
         }
         else
         {
-            res = load_path(path, &stbuf, buf, filler);
+            res = load_path(_path, &stbuf, buf, filler);
         }
 
         add_dotdot(buf, filler, &stbuf, 0);
